#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2025, Samita Bhattacharjee (@samiib) <samitab@cisco.com>

# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {
    "metadata_version": "1.1",
    "status": ["preview"],
    "supported_by": "community",
}

DOCUMENTATION = r"""
---
module: ndo_tenant_span_session
version_added: "2.11.0"
short_description: Manage Tenant SPAN Sessions on Cisco Nexus Dashboard Orchestrator (NDO).
description:
- Manage Switched Port Analyzer (SPAN) Sessions on Cisco Nexus Dashboard Orchestrator (NDO).
- This module is only supported on ND v3.1 (NDO v4.3) and later.
author:
- Samita Bhattacharjee (@samiib)
options:
  template:
    description:
    - The name of the template.
    - The template must be a Tenant Monitoring Policy template.
    - This parameter or O(template_id) is required.
    type: str
  template_id:
    description:
    - The ID of the template.
    - The template must be a Tenant Monitoring Policy template.
    - This parameter or O(template) is required.
    type: str
  name:
    description:
    - The name of the SPAN Session.
    type: str
    aliases: [ span_session ]
  uuid:
    description:
    - The UUID of the SPAN Session.
    - This parameter is required when the O(name) needs to be updated.
    type: str
    aliases: [ span_session_uuid ]
  description:
    description:
    - The description of the SPAN Session.
    type: str
  sources:
    description:
    - The SPAN Session sources.
    - Providing a new list of O(sources) will replace the existing sources from the SPAN Session.
    - Providing an empty list will remove the O(sources=[]) from the SPAN Session.
    type: list
    elements: dict
    suboptions:
      name:
        description:
        - The name of the SPAN Session source.
        type: str
        required: true
      direction:
        description:
        - The direction of the SPAN Session source.
        type: str
        required: true
        choices: [ incoming, outgoing, both ]
      epg_uuid:
        description:
        - The UUID of the source Endpoint Group (EPG) to use for the SPAN Session.
        - This parameter or O(sources.epg) is required.
        type: str
      epg:
        description:
        - The EPG to use for the SPAN Session source.
        - This parameter or O(sources.epg_uuid) is required.
        type: dict
        suboptions:
          name:
            description:
            - The name of the source EPG.
            type: str
            required: true
          template:
            description:
            - The name of the template that contains the source EPG.
            - This parameter or O(sources.epg.template_id) is required.
            type: str
          template_id:
            description:
            - The ID of the template that contains the source EPG.
            - This parameter or O(sources.epg.template) is required.
            type: str
          schema_id:
            description:
            - The ID of the schema that contains the source EPG.
            - This parameter or O(sources.epg.schema) is required.
            type: str
          schema:
            description:
            - The name of the schema that contains the source EPG.
            - This parameter or O(sources.epg.schema_id) is required.
            type: str
          anp:
            description:
            - The name of the Application Profile (ANP) that contains the source EPG.
            - This parameter or O(sources.epg.anp_uuid) is required.
            type: str
          anp_uuid:
            description:
            - The UUID of the ANP that contains the source EPG.
            - This parameter or O(sources.epg.anp) is required.
            type: str
  admin_state:
    description:
    - The administrative state of the SPAN Session.
    - Defaults to C(enabled) when unset during creation.
    choices: [ enabled, disabled ]
    type: str
  mtu:
    description:
    - The MTU truncation size for the SPAN packets.
    - The value must be in the range 64 - 9216.
    - Defaults to 1518 when unset during creation.
    type: int
  destination_epg:
    description:
    - The destination EPG configuration group.
    type: dict
    suboptions:
      epg_uuid:
        description:
        - The UUID of the destination EPG to use for the SPAN Session.
        - This parameter or O(destination_epg.epg) is required when creating the SPAN Session.
        type: str
      epg:
        description:
        - The destination EPG to use for the SPAN Session.
        - This parameter or O(destination_epg.epg_uuid) is required when creating the SPAN Session.
        type: dict
        suboptions:
          name:
            description:
            - The name of the destination EPG.
            type: str
            required: true
          template:
            description:
            - The name of the template that contains the destination EPG.
            - This parameter or O(destination_epg.epg.template_id) is required.
            type: str
          template_id:
            description:
            - The ID of the  that contains the destination EPG.
            - This parameter or O(destination_epg.epg.template) is required.
            type: str
          schema_id:
            description:
            - The ID of the schema that contains the destination EPG.
            - This parameter or O(destination_epg.epg.schema) is required.
            type: str
          schema:
            description:
            - The name of the schema that contains the destination EPG.
            - This parameter or O(destination_epg.epg.schema_id) is required.
            type: str
          anp:
            description:
            - The name of the ANP that contains the destination EPG.
            - This parameter or O(destination_epg.epg.anp_uuid) is required.
            type: str
          anp_uuid:
            description:
            - The UUID of the ANP that contains the destination EPG.
            - This parameter or O(destination_epg.epg.anp) is required.
            type: str
      destination_ip:
        description:
        - The destination IP address to route SPAN Session packets.
        type: str
      source_ip_prefix:
        description:
        - The prefix used to assign source IP addresses to ERSPAN packets which can be used to identify which Leaf or Spine is sending the traffic.
        - This can be any IP. If the prefix is used, the node ID of the source node is used for the undefined bits of the prefix.
        type: str
      span_version:
        description:
        - The version of the SPAN Session.
        - Defaults to C(v2) when unset during creation.
        choices: [ v1, v2 ]
        type: str
      enforce_span_version:
        description:
        - Enforce the SPAN Session version defined in O(destination_epg.span_version).
        - Defaults to true when unset during creation.
        type: bool
      flow_id:
        description:
        - The flow ID of the SPAN Session packets.
        - The value must be in the range 1 - 1023.
        - Defaults to 1 when unset during creation.
        type: int
      ttl:
        description:
        - The time to live (TTL) of the SPAN Session packets.
        - The value must be in the range 1 - 1023.
        - Defaults to 1 when unset during creation.
        type: int
      dscp:
        description:
        - The DSCP value for sending the monitored SPAN Session packets.
        - Defaults to C(unspecified) when unset during creation.
        choices:
          - af11
          - af12
          - af13
          - af21
          - af22
          - af23
          - af31
          - af32
          - af33
          - af41
          - af42
          - af43
          - cs0
          - cs1
          - cs2
          - cs3
          - cs4
          - cs5
          - cs6
          - cs7
          - expedited_forwarding
          - voice_admit
          - unspecified
        type: str
  state:
    description:
    - Use C(absent) for removing.
    - Use C(query) for listing an object or multiple objects.
    - Use C(present) for creating or updating.
    type: str
    choices: [ absent, query, present ]
    default: query
notes:
- The O(template), O(destination_epg.epg) and O(sources.epg) must exist before using this module in your playbook.
  Use M(cisco.mso.ndo_template) to create the Tenant Monitoring Policy template.
  Use M(cisco.mso.mso_schema_template_anp_epg) to create the EPGs.
seealso:
- module: cisco.mso.ndo_template
- module: cisco.mso.mso_schema_template_anp_epg
extends_documentation_fragment: cisco.mso.modules
"""

EXAMPLES = r"""
- name: Create a new SPAN Session
  cisco.mso.ndo_tenant_span_session:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: monitoring_tenant_template
    name: span_session_1
    sources:
      - name: source_1
        epg_uuid: '{{ query_epg_1.current.uuid }}'
        direction: both
      - name: source_2
        direction: outgoing
        epg:
          name: epg_2
          anp: anp_1
          template: template_1
          schema: schema_1
    admin_state: enabled
    mtu: 9216
    destination_epg:
      epg:
        name: epg_3
        anp: anp_1
        template: template_1
        schema: schema_1
      destination_ip: 10.1.1.1
      source_ip_prefix: 10.1.1.1/24
      span_version: v1
      enforce_span_version: false
      flow_id: 15
      ttl: 128
      dscp: af11
    state: present
  register: create_span_session_1

- name: Update the name of the SPAN Session using UUID
  cisco.mso.ndo_tenant_span_session:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template_id: "{{ create_monitoring_tenant.current.templateId }}"
    uuid: "{{ create_span_session_1.current.uuid }}"
    name: span_session_1_updated
    state: present
  register: update_span_session_1

- name: Query an existing SPAN Session using UUID
  cisco.mso.ndo_tenant_span_session:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: monitoring_tenant_template
    uuid: "{{ create_span_session_1.current.uuid }}"
    state: query
  register: query_with_uuid

- name: Query an existing SPAN Session using name
  cisco.mso.ndo_tenant_span_session:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: monitoring_tenant_template
    name: span_session_1_updated
    state: query
  register: query_with_name

- name: Query all SPAN Sessions
  cisco.mso.ndo_tenant_span_session:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: monitoring_tenant_template
    state: query
  register: query_all

- name: Remove all sources from a SPAN Session
  cisco.mso.ndo_tenant_span_session:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: monitoring_tenant_template
    name: span_session_1_updated
    sources: []
    state: present

- name: Delete an existing SPAN Session using UUID
  cisco.mso.ndo_tenant_span_session:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: monitoring_tenant_template
    uuid: "{{ create_span_session_1.current.uuid }}"
    state: absent

- name: Delete an existing SPAN Session using Name
  cisco.mso.ndo_tenant_span_session:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: monitoring_tenant_template
    name: span_session_1_updated
    state: absent
"""

RETURN = r"""
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.mso.plugins.module_utils.mso import MSOModule, mso_argument_spec, epg_object_reference_spec
from ansible_collections.cisco.mso.plugins.module_utils.schemas import MSOSchemas
from ansible_collections.cisco.mso.plugins.module_utils.template import MSOTemplate, KVPair
from ansible_collections.cisco.mso.plugins.module_utils.constants import TARGET_DSCP_MAP, ENABLED_OR_DISABLED_TO_BOOL_STRING_MAP
from ansible_collections.cisco.mso.plugins.module_utils.utils import append_update_ops_data
import copy


def main():
    argument_spec = mso_argument_spec()
    argument_spec.update(
        template=dict(type="str"),
        template_id=dict(type="str"),
        name=dict(type="str", aliases=["span_session"]),
        uuid=dict(type="str", aliases=["span_session_uuid"]),
        description=dict(type="str"),
        sources=dict(
            type="list",
            elements="dict",
            mutually_exclusive=[
                ("epg", "epg_uuid"),
            ],
            required_one_of=[
                ["epg", "epg_uuid"],
            ],
            options=dict(
                name=dict(type="str", required=True),
                direction=dict(type="str", required=True, choices=["incoming", "outgoing", "both"]),
                epg_uuid=dict(type="str"),
                epg=epg_object_reference_spec(),
            ),
        ),
        admin_state=dict(type="str", choices=list(ENABLED_OR_DISABLED_TO_BOOL_STRING_MAP)),
        mtu=dict(type="int"),
        destination_epg=dict(
            type="dict",
            mutually_exclusive=[
                ("epg", "epg_uuid"),
            ],
            options=dict(
                epg_uuid=dict(type="str"),
                epg=epg_object_reference_spec(),
                destination_ip=dict(type="str"),
                source_ip_prefix=dict(type="str"),
                span_version=dict(type="str", choices=["v1", "v2"]),
                enforce_span_version=dict(type="bool"),
                flow_id=dict(type="int"),
                ttl=dict(type="int"),
                dscp=dict(type="str", choices=list(TARGET_DSCP_MAP)),
            ),
        ),
        state=dict(type="str", default="query", choices=["absent", "query", "present"]),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        mutually_exclusive=[
            ("template", "template_id"),
        ],
        required_if=[
            ["state", "absent", ["name", "uuid"], True],
            ["state", "present", ["name", "uuid"], True],
        ],
        required_one_of=[["template", "template_id"]],
    )

    mso = MSOModule(module)
    mso_schemas = MSOSchemas(mso)

    template_name = module.params.get("template")
    template_id = module.params.get("template_id")
    name = module.params.get("name")
    uuid = module.params.get("uuid")
    description = module.params.get("description")
    sources = module.params.get("sources")
    admin_state = module.params.get("admin_state")
    mtu = module.params.get("mtu")
    destination_epg = module.params.get("destination_epg")
    state = module.params.get("state")

    ops = []
    match = None
    reference_collection = {
        "epgRef": {
            "name": "epgName",
            "reference": "epgRef",
            "type": "epg",
            "template": "epgTemplateName",
            "templateId": "epgTemplateId",
            "schema": "epgSchemaName",
            "schemaId": "epgSchemaId",
        },
        "epg": {
            "name": "epgName",
            "reference": "epg",
            "type": "epg",
            "template": "epgTemplateName",
            "templateId": "epgTemplateId",
            "schema": "epgSchemaName",
            "schemaId": "epgSchemaId",
        },
    }

    mso_template = MSOTemplate(mso, "monitoring_tenant", template_name, template_id)
    mso_template.validate_template("monitoring")
    object_description = "SPAN Session"
    path = "/monitoringTemplate/template/spanSessions"
    span_session_path = None

    existing_span_sessions = mso_template.template.get("monitoringTemplate", {}).get("template", {}).get("spanSessions") or []

    if name or uuid:
        match = mso_template.get_object_by_key_value_pairs(
            object_description,
            existing_span_sessions,
            [KVPair("uuid", uuid) if uuid else KVPair("name", name)],
        )
        if match:
            span_session_path = "{0}/{1}".format(path, match.index)
            mso_template.update_config_with_template_and_references(match.details, reference_collection)
            mso.existing = mso.previous = copy.deepcopy(match.details)
    else:
        mso.existing = mso.previous = [mso_template.update_config_with_template_and_references(obj, reference_collection) for obj in existing_span_sessions]

    if state == "present":
        if uuid and not mso.existing:
            mso.fail_json(msg="{0} with the UUID: '{1}' not found".format(object_description, uuid))

        mso_values = dict(name=name, description=description, destination=dict(mtu=mtu))
        if admin_state is not None or sources is not None:
            source_group = dict()
            if admin_state is not None:
                source_group["enableAdminState"] = ENABLED_OR_DISABLED_TO_BOOL_STRING_MAP.get(admin_state)
            if sources is not None:
                source_group["sources"] = format_sources(mso_schemas, sources)
            mso_values["sourceGroup"] = source_group
        if destination_epg:
            mso_values["destination"]["remote"] = dict(
                epgRef=mso_schemas.get_epg_uuid(destination_epg.get("epg"), destination_epg.get("epg_uuid")),
                spanVersion=destination_epg.get("span_version"),
                enforceSpanVersion=destination_epg.get("enforce_span_version"),
                destIPAddress=destination_epg.get("destination_ip"),
                srcIPPrefix=destination_epg.get("source_ip_prefix"),
                flowID=destination_epg.get("flow_id"),
                ttl=destination_epg.get("ttl"),
                dscp=TARGET_DSCP_MAP.get(destination_epg.get("dscp")),
            )

        if match:
            mso_update_values = {
                "name": name,
                "description": description,
                ("destination", "mtu"): mtu,
            }
            source_group = mso_values.get("sourceGroup")
            if source_group:
                mso_update_values[("sourceGroup", "enableAdminState")] = source_group.get("enableAdminState")
                mso_update_values[("sourceGroup", "sources")] = source_group.get("sources")
            if destination_epg:
                remote_group = mso_values.get("destination", {}).get("remote")
                if remote_group:
                    mso_update_values[("destination", "remote", "epgRef")] = remote_group.get("epgRef")
                    mso_update_values[("destination", "remote", "spanVersion")] = remote_group.get("spanVersion")
                    mso_update_values[("destination", "remote", "enforceSpanVersion")] = remote_group.get("enforceSpanVersion")
                    mso_update_values[("destination", "remote", "destIPAddress")] = remote_group.get("destIPAddress")
                    mso_update_values[("destination", "remote", "srcIPPrefix")] = remote_group.get("srcIPPrefix")
                    mso_update_values[("destination", "remote", "flowID")] = remote_group.get("flowID")
                    mso_update_values[("destination", "remote", "ttl")] = remote_group.get("ttl")
                    mso_update_values[("destination", "remote", "dscp")] = remote_group.get("dscp")

            proposed_payload = copy.deepcopy(match.details)
            append_update_ops_data(ops, proposed_payload, span_session_path, mso_update_values)
            mso.sanitize(proposed_payload, collate=True)
        else:
            mso.sanitize(mso_values)
            ops.append(dict(op="add", path="{0}/-".format(path), value=mso.sent))

    elif state == "absent":
        if match:
            ops.append(dict(op="remove", path=span_session_path))

    if not module.check_mode and ops:
        response = mso.request(mso_template.template_path, method="PATCH", data=ops)
        node_profiles = response.get("monitoringTemplate", {}).get("template", {}).get("spanSessions") or []
        match = mso_template.get_object_by_key_value_pairs(
            object_description,
            node_profiles,
            [KVPair("uuid", uuid) if uuid else KVPair("name", name)],
        )
        if match:
            mso_template.update_config_with_template_and_references(match.details, reference_collection)
            mso.existing = match.details  # When the state is present
        else:
            mso.existing = {}  # When the state is absent
    elif module.check_mode and state != "query":  # When the state is present/absent with check mode
        mso_template.update_config_with_template_and_references(mso.proposed, reference_collection)
        mso.existing = mso.proposed if state == "present" else {}

    mso.exit_json()


def format_sources(schemas, sources):
    source_list = []
    for source in sources:
        source_values = {
            "name": source.get("name"),
            "direction": source.get("direction"),
            "epg": schemas.get_epg_uuid(source.get("epg"), source.get("epg_uuid")),
        }
        source_list.append(source_values)
    return source_list


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2024, Sabari Jaganathan (@sajagana) <sajagana@cisco.com>

# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "community"}

DOCUMENTATION = r"""
---
module: ndo_l3out_node_group_policy
short_description: Manage L3Out Node Group Policy on Cisco Nexus Dashboard Orchestrator (NDO).
description:
- Manage L3Out Node Group Policy on Cisco Nexus Dashboard Orchestrator (NDO).
- This module is only supported on ND v3.1 (NDO v4.3) and later.
author:
- Sabari Jaganathan (@sajagana)
options:
  template:
    description:
    - The name of the template.
    - The template must be an L3Out template.
    type: str
    aliases: [ l3out_template ]
    required: true
  l3out:
    description:
    - The name of the L3Out.
    type: str
    aliases: [ l3out_name ]
    required: true
  name:
    description:
    - The name of the L3Out Node Group Policy.
    type: str
    aliases: [ l3out_node_group_policy ]
  description:
    description:
    - The description of the L3Out Node Group Policy.
    type: str
  node_routing_policy:
    description:
    - The name of the L3Out Node Routing Policy.
    - Providing an empty string will remove the O(node_routing_policy="") from L3Out Node Group Policy.
    type: str
  bfd:
    description:
    - The Bidirectional Forwarding Detection (BFD) multi-hop configuration of the L3Out Node Group Policy.
    - Providing an empty dictionary will remove the O(bfd={}) from the L3Out Node Group Policy.
    type: dict
    suboptions:
      auth:
        description:
        - The BFD multi-hop authentication of the L3Out Node Group Policy.
        type: str
        choices: [ enabled, disabled ]
        aliases: [ bfd_multi_hop_authentication ]
      key_id:
        description:
        - The BFD multi-hop key ID of the L3Out Node Group Policy.
        type: int
      key:
        description:
        - The BFD multi-hop key of the L3Out Node Group Policy.
        type: str
    aliases: [ bfd_multi_hop ]
  target_dscp:
    description:
    - The DSCP Level of the L3Out Node Group Policy.
    type: str
    choices:
      - af11
      - af12
      - af13
      - af21
      - af22
      - af23
      - af31
      - af32
      - af33
      - af41
      - af42
      - af43
      - cs0
      - cs1
      - cs2
      - cs3
      - cs4
      - cs5
      - cs6
      - cs7
      - expedited_forwarding
      - unspecified
      - voice_admit
  state:
    description:
    - Use C(absent) for removing.
    - Use C(query) for listing an object or multiple objects.
    - Use C(present) for creating or updating.
    type: str
    choices: [ absent, query, present ]
    default: query
notes:
- The O(template) must exist before using this module in your playbook.
  Use M(cisco.mso.ndo_template) to create the L3Out template.
- The O(l3out) must exist before using this module in your playbook.
  Use M(cisco.mso.ndo_l3out_template) to create the L3Out object under the L3Out template.
- The O(node_routing_policy) must exist before using this module in your playbook.
  Use M(cisco.mso.ndo_l3out_node_routing_policy) to create the L3Out Node Routing Policy.
seealso:
- module: cisco.mso.ndo_template
- module: cisco.mso.ndo_l3out_template
- module: cisco.mso.ndo_l3out_node_routing_policy
extends_documentation_fragment: cisco.mso.modules
"""

EXAMPLES = r"""
- name: Create a new L3Out node group policy
  cisco.mso.ndo_l3out_node_group_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: l3out_template
    l3out: l3out
    name: node_group_policy_1
    state: present

- name: Update an existing L3Out node group policy
  cisco.mso.ndo_l3out_node_group_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: l3out_template
    l3out: l3out
    name: node_group_policy_1
    description: "Updated description"
    node_routing_policy: ans_node_policy_group_1
    bfd:
      auth: enabled
      key_id: 1
      key: TestKey
    target_dscp: af11
    state: present

- name: Query an existing L3Out node group policy with name
  cisco.mso.ndo_l3out_node_group_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: l3out_template
    l3out: l3out
    name: node_group_policy_1
    state: query
  register: query_with_name

- name: Query all L3Out node group policy
  cisco.mso.ndo_l3out_node_group_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: l3out_template
    l3out: l3out
    state: query
  register: query_all

- name: Delete an existing L3Out node group policy with name
  cisco.mso.ndo_l3out_node_group_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: l3out_template
    l3out: l3out
    name: node_group_policy_1
    state: absent
"""

RETURN = r"""
"""


import copy
from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.mso.plugins.module_utils.mso import MSOModule, mso_argument_spec
from ansible_collections.cisco.mso.plugins.module_utils.template import MSOTemplate, KVPair
from ansible_collections.cisco.mso.plugins.module_utils.constants import TARGET_DSCP_MAP, ENABLED_OR_DISABLED_TO_BOOL_STRING_MAP
from ansible_collections.cisco.mso.plugins.module_utils.utils import generate_api_endpoint, check_if_all_elements_are_none, append_update_ops_data


def bfd_multi_hop_mso_values(bfd):
    return dict(
        authEnabled=ENABLED_OR_DISABLED_TO_BOOL_STRING_MAP.get(bfd.get("auth")),
        keyID=bfd.get("key_id"),
        key=dict(
            value=bfd.get("key"),
        ),
    )


def main():
    argument_spec = mso_argument_spec()
    argument_spec.update(
        template=dict(type="str", required=True, aliases=["l3out_template"]),
        l3out=dict(type="str", required=True, aliases=["l3out_name"]),
        name=dict(type="str", aliases=["l3out_node_group_policy"]),
        description=dict(type="str"),
        node_routing_policy=dict(type="str"),
        bfd=dict(
            type="dict",
            options=dict(
                auth=dict(type="str", choices=["enabled", "disabled"], aliases=["bfd_multi_hop_authentication"]),
                key_id=dict(type="int"),
                key=dict(type="str", no_log=True),
            ),
            aliases=["bfd_multi_hop"],
        ),
        target_dscp=dict(type="str", choices=list(TARGET_DSCP_MAP)),
        state=dict(type="str", default="query", choices=["absent", "query", "present"]),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "absent", ["name"]],
            ["state", "present", ["name"]],
        ],
    )

    mso = MSOModule(module)

    template = module.params.get("template")
    l3out = module.params.get("l3out")
    name = module.params.get("name")
    description = module.params.get("description")
    node_routing_policy = module.params.get("node_routing_policy")
    bfd = module.params.get("bfd")
    target_dscp = TARGET_DSCP_MAP.get(module.params.get("target_dscp"))
    state = module.params.get("state")

    mso_template = MSOTemplate(mso, "l3out", template)
    mso_template.validate_template("l3out")

    l3out_object = mso_template.get_l3out_object(name=l3out, fail_module=True)
    l3out_node_group = mso_template.get_l3out_node_group(name, l3out_object.details)

    if name and l3out_node_group:
        mso.existing = copy.deepcopy(l3out_node_group.details)
        mso.previous = copy.deepcopy(l3out_node_group.details)  # Query a specific object
    elif l3out_node_group:
        mso.existing = l3out_node_group  # Query all objects

    if state != "query":
        node_group_policy_path = "/l3outTemplate/l3outs/{0}/nodeGroups/{1}".format(l3out_object.index, l3out_node_group.index if l3out_node_group else "-")

    ops = []

    if state == "present":
        if bfd:
            # True if all elements are None, False otherwise.
            bfd_is_empty = check_if_all_elements_are_none(list(bfd.values()))

        l3out_node_routing_policy_object = None
        if node_routing_policy:
            l3out_node_routing_policy_objects = mso.query_objs(
                generate_api_endpoint(
                    "templates/objects", **{"type": "l3OutNodePolGroup", "tenant-id": mso_template.template_summary.get("tenantId"), "include-common": "true"}
                )
            )
            l3out_node_routing_policy_object = mso_template.get_object_by_key_value_pairs(
                "L3Out Node Routing Policy", l3out_node_routing_policy_objects, [KVPair("name", node_routing_policy)], True
            )

        if mso.existing:
            mso_values = dict()
            mso_values_remove = list()

            if mso.existing.get("bfdMultiHop", {}).get("key", {}).get("ref"):
                mso.existing["bfdMultiHop"]["key"].pop("ref", None)
                mso.previous["bfdMultiHop"]["key"].pop("ref", None)

            mso_values["description"] = description

            if node_routing_policy == "" and mso.existing.get(
                "nodeRoutingPolicyRef"
            ):  # Clear the node routing policy when node_routing_policy is empty string
                mso_values_remove.append("nodeRoutingPolicyRef")
            else:
                mso_values["nodeRoutingPolicyRef"] = l3out_node_routing_policy_object.details.get("uuid") if l3out_node_routing_policy_object else None

            if bfd:
                if bfd_is_empty and mso.existing.get("bfdMultiHop"):
                    mso_values_remove.append("bfdMultiHop")

                elif not bfd_is_empty and not mso.existing.get("bfdMultiHop"):
                    mso_values["bfdMultiHop"] = bfd_multi_hop_mso_values(bfd)

                elif not bfd_is_empty and mso.existing.get("bfdMultiHop"):
                    mso_values[("bfdMultiHop", "authEnabled")] = ENABLED_OR_DISABLED_TO_BOOL_STRING_MAP.get(bfd.get("auth"))
                    mso_values[("bfdMultiHop", "keyID")] = bfd.get("key_id")

                    if bfd.get("key") is not None:
                        if mso.existing.get("bfdMultiHop", {}).get("key") is not None:
                            mso_values[("bfdMultiHop", "key", "value")] = bfd.get("key")
                        else:
                            mso_values[("bfdMultiHop", "key")] = dict(value=bfd.get("key"))

            mso_values["targetDscp"] = target_dscp

            append_update_ops_data(ops, mso.existing, node_group_policy_path, mso_values, mso_values_remove)
            mso.sanitize(mso.existing, collate=True)
        else:
            mso_values = dict(name=name)
            mso_values["description"] = description

            if l3out_node_routing_policy_object:
                mso_values["nodeRoutingPolicyRef"] = l3out_node_routing_policy_object.details.get("uuid")

            if bfd and not bfd_is_empty:
                mso_values["bfdMultiHop"] = bfd_multi_hop_mso_values(bfd)

            mso_values["targetDscp"] = target_dscp

            mso.sanitize(mso_values)
            ops.append(dict(op="add", path=node_group_policy_path, value=mso_values))

        mso.existing = mso.proposed

    elif state == "absent":
        if mso.existing:
            ops.append(dict(op="remove", path=node_group_policy_path))

    if not module.check_mode and ops:
        mso_template.template = mso.request(mso_template.template_path, method="PATCH", data=ops)
        l3out_object = mso_template.get_l3out_object(name=l3out, fail_module=True)
        l3out_node_group = mso_template.get_l3out_node_group(name, l3out_object.details)
        if l3out_node_group:
            mso.existing = l3out_node_group.details  # When the state is present
            if mso.existing.get("bfdMultiHop", {}).get("key", {}).get("ref"):
                mso.existing["bfdMultiHop"]["key"].pop("ref")
        else:
            mso.existing = {}  # When the state is absent
    elif module.check_mode and state != "query":  # When the state is present/absent with check mode
        mso.existing = mso.proposed if state == "present" else {}

    mso.exit_json()


if __name__ == "__main__":
    main()



# Copyright (c) 2019-2021, Felix Fontein <felix@fontein.de>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

# Note that this module util is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

from __future__ import annotations

import fcntl
import os
import os.path
import socket as pysocket
import typing as t
from collections.abc import Callable

if t.TYPE_CHECKING:
    SocketLike = pysocket.socket


def make_file_unblocking(file: SocketLike) -> None:
    fcntl.fcntl(
        file.fileno(),
        fcntl.F_SETFL,
        fcntl.fcntl(file.fileno(), fcntl.F_GETFL) | os.O_NONBLOCK,
    )


def make_file_blocking(file: SocketLike) -> None:
    fcntl.fcntl(
        file.fileno(),
        fcntl.F_SETFL,
        fcntl.fcntl(file.fileno(), fcntl.F_GETFL) & ~os.O_NONBLOCK,
    )


def make_unblocking(sock: SocketLike) -> None:
    if hasattr(sock, "_sock"):
        sock._sock.setblocking(0)
    elif hasattr(sock, "setblocking"):
        sock.setblocking(0)  # type: ignore  # TODO: CHECK!
    else:
        make_file_unblocking(sock)


def _empty_writer(msg: str) -> None:
    pass


def shutdown_writing(
    sock: SocketLike, log: Callable[[str], None] = _empty_writer
) -> None:
    # FIXME: This does **not work with SSLSocket**! Apparently SSLSocket does not allow to send
    #        a close_notify TLS alert without completely shutting down the connection.
    #        Calling sock.shutdown(pysocket.SHUT_WR) simply turns of TLS encryption and from that
    #        point on the raw encrypted data is returned when sock.recv() is called. :-(
    if hasattr(sock, "shutdown_write"):
        sock.shutdown_write()
    elif hasattr(sock, "shutdown"):
        try:
            sock.shutdown(pysocket.SHUT_WR)
        except TypeError as e:
            # probably: "TypeError: shutdown() takes 1 positional argument but 2 were given"
            log(f"Shutting down for writing not possible; trying shutdown instead: {e}")
            sock.shutdown()  # type: ignore
    elif isinstance(sock, pysocket.SocketIO):  # type: ignore
        sock._sock.shutdown(pysocket.SHUT_WR)  # type: ignore[unreachable]
    else:
        log("No idea how to signal end of writing")


def write_to_socket(sock: SocketLike, data: bytes) -> int:
    if hasattr(sock, "_send_until_done"):
        # WrappedSocket (urllib3/contrib/pyopenssl) does not have `send`, but
        # only `sendall`, which uses `_send_until_done` under the hood.
        return sock._send_until_done(data)
    if hasattr(sock, "send"):
        return sock.send(data)
    return os.write(sock.fileno(), data)



# Copyright (c) 2023, Felix Fontein <felix@fontein.de>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
name: gpg_fingerprint
short_description: Retrieve a GPG fingerprint from a GPG public or private key file
author: Felix Fontein (@felixfontein)
version_added: 2.15.0
description:
  - Takes a list of filenames pointing to GPG public or private key files. Returns the fingerprints for each of these keys.
options:
  _terms:
    description:
      - A path to a GPG public or private key.
    type: list
    elements: path
    required: true
requirements:
  - GnuPG (C(gpg) executable)
seealso:
  - plugin: community.crypto.gpg_fingerprint
    plugin_type: filter
"""

EXAMPLES = r"""
---
- name: Show fingerprint of GPG public key
  ansible.builtin.debug:
    msg: "{{ lookup('community.crypto.gpg_fingerprint', '/path/to/public_key.gpg') }}"
"""

RETURN = r"""
_value:
  description:
    - The fingerprints of the provided public or private GPG keys.
    - The list has one entry for every path provided.
  type: list
  elements: string
"""

import os
import typing as t

from ansible.errors import AnsibleLookupError
from ansible.module_utils.common.text.converters import to_text
from ansible.plugins.lookup import LookupBase

from ansible_collections.community.crypto.plugins.module_utils._gnupg.cli import (
    GPGError,
    get_fingerprint_from_file,
)
from ansible_collections.community.crypto.plugins.plugin_utils._gnupg import (
    PluginGPGRunner,
)


class LookupModule(LookupBase):
    def run(
        self, terms: list[t.Any], variables: None = None, **kwargs: t.Any
    ) -> list[str]:
        self.set_options(direct=kwargs)
        if self._loader is None:
            raise AssertionError(
                "Contract violation: self._loader is None"
            )  # pragma: no cover

        try:
            gpg = PluginGPGRunner(cwd=self._loader.get_basedir())
            result = []
            for i, path in enumerate(terms):
                if not isinstance(path, (str, bytes, os.PathLike)):
                    raise AnsibleLookupError(
                        f"Lookup parameter #{i} should be string or a path object, but got {type(path)}"
                    )
                result.append(
                    get_fingerprint_from_file(gpg_runner=gpg, path=to_text(path))
                )
            return result
        except GPGError as exc:
            raise AnsibleLookupError(str(exc)) from exc



#!/usr/bin/python
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
#
# Python SDK Reference: https://learn.microsoft.com/en-us/python/api/azure-mgmt-cdn/azure.mgmt.cdn.operations.afdrulesetsoperations?view=azure-python
#

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_afdruleset_info
version_added: "3.4.0"
short_description: Get Azure Front Door Ruleset facts to be used with Standard or Premium Frontdoor Service
description:
    - Get facts for a specific Azure Front Door (AFD) Ruleset or all AFD Rulesets.
    - This differs from the Front Door classic service and only is intended to be used by the Standard or Premium service offering.

options:
    name:
        description:
            - Limit results to a specific AFD Ruleset.
        type: str
    profile_name:
        description:
            - Name of the Azure Front Door Standard or Azure Front Door Premium profile which is unique within the resource group.
        required: true
        type: str
    resource_group:
        description:
            - Name of the Resource group within the Azure subscription.
        required: true
        type: str

extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - Jarret Tooley (@jartoo)
'''

EXAMPLES = '''
- name: Get facts for all Rulesets in AFD Profile
  azure_rm_afdruleset_info:
    resource_group: myResourceGroup
    profile_name: myProfile

- name: Get facts of specific AFD Ruleset
  azure_rm_afdruleset_info:
    name: myRuleset
    profile_name: myProfile
    resource_group: myResourceGroup
'''

RETURN = '''
afdrulesets:
    description: List of AFD Rulesets.
    returned: always
    type: complex
    contains:
        deployment_status:
            description:
                - Current state of the resource.
            type: str
            sample: NotStarted
        id:
            description:
                - ID of the AFD Ruleset.
            type: str
            sample: "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourcegroups/myRG/providers/Microsoft.Cdn/profiles/myProf/rulesets/myRS"
        name:
            description:
                - Name of the AFD Ruleset.
            type: str
        profile_name:
            description:
                - The name of the profile which holds the rule set.
            type: str
        provisioning_state:
            description:
                - Provisioning status.
            type: str
            sample: Succeeded
        resource_group:
            description:
                - Name of a resource group.
            type: str
        type:
            description:
                - Resource type.
            type: str
            sample: Microsoft.Cdn/profiles/afdrulesets
'''

import re
from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase


AZURE_OBJECT_CLASS = 'Ruleset'


class AzureRMAFDRulesetInfo(AzureRMModuleBase):
    """Utility class to get Azure AFD Ruleset facts"""

    def __init__(self):

        self.module_args = dict(
            name=dict(
                type='str'
            ),
            resource_group=dict(
                type='str',
                required=True
            ),
            profile_name=dict(
                type='str',
                required=True
            )
        )

        self.results = dict(
            changed=False,
            afdrulesets=[]
        )

        self.name = None
        self.profile_name = None
        self.resource_group = None

        super(AzureRMAFDRulesetInfo, self).__init__(
            supports_check_mode=True,
            derived_arg_spec=self.module_args,
            supports_tags=False,
            facts_module=True
        )

    def exec_module(self, **kwargs):

        for key in self.module_args:
            setattr(self, key, kwargs[key])

        if self.name:
            self.results['afdrulesets'] = self.get_item()
        else:
            self.results['afdrulesets'] = self.list_by_profile()

        return self.results

    def get_item(self):
        """Get a single Azure AFD Ruleset"""

        self.log('Get properties for {0}'.format(self.name))

        item = None
        result = []

        try:
            item = self.endpoint_client.rule_sets.get(
                resource_group_name=self.resource_group,
                profile_name=self.profile_name,
                rule_set_name=self.name)
        except Exception:
            pass

        if item:
            result = [self.serialize_afdruleset(item)]

        return result

    def list_by_profile(self):
        """Get all Azure AFD Rulesets within an AFD profile"""

        self.log('List all AFD Rulesets within an AFD profile')

        try:
            response = self.cdn_client.rule_sets.list_by_profile(
                resource_group_name=self.resource_group,
                profile_name=self.profile_name)
        except Exception as exc:
            self.fail('Failed to list all items - {0}'.format(str(exc)))

        results = []
        for item in response:
            results.append(self.serialize_afdruleset(item))

        return results

    def serialize_afdruleset(self, afdruleset):
        '''
        Convert a AFD Ruleset object to dict.
        :param afdruleset: AFD Ruleset object
        :return: dict
        '''
        result = self.serialize_obj(afdruleset, AZURE_OBJECT_CLASS)

        new_result = {}
        new_result['deployment_status'] = afdruleset.deployment_status
        new_result['id'] = afdruleset.id
        new_result['name'] = afdruleset.name
        new_result['profile_name'] = re.sub('\\/.*', '', re.sub('.*profiles\\/', '', result['id']))
        new_result['provisioning_state'] = afdruleset.provisioning_state
        new_result['resource_group'] = re.sub('\\/.*', '', re.sub('.*resourcegroups\\/', '', result['id']))
        new_result['type'] = afdruleset.type
        return new_result


def main():
    """Main module execution code path"""
    AzureRMAFDRulesetInfo()


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-

# Copyright (c) 2021 Ansible Project
#
# This code is part of Ansible, but is an independent component.
# This particular file snippet, and this file snippet only, is BSD licensed.
# Modules you write using this snippet, which is embedded dynamically by Ansible
# still belong to the author of the module, and may assign their own license
# to the complete work.
#
# Redistribution and use in source and binary forms, with or without modification,
# are permitted provided that the following conditions are met:
#
#    * Redistributions of source code must retain the above copyright
#      notice, this list of conditions and the following disclaimer.
#    * Redistributions in binary form must reproduce the above copyright notice,
#      this list of conditions and the following disclaimer in the documentation
#      and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
# IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
# USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import functools
import random
import time


class BackoffIterator:
    """iterate sleep value based on the exponential or jitter back-off algorithm.
    Args:
        delay (int or float): initial delay.
        backoff (int or float): backoff multiplier e.g. value of 2 will  double the delay each retry.
        max_delay (int or None): maximum amount of time to wait between retries.
        jitter (bool): if set to true, add jitter to the generate value.
    """

    def __init__(self, delay, backoff, max_delay=None, jitter=False):
        self.delay = delay
        self.backoff = backoff
        self.max_delay = max_delay
        self.jitter = jitter

    def __iter__(self):
        self.current_delay = self.delay
        return self

    def __next__(self):
        return_value = self.current_delay if self.max_delay is None else min(self.current_delay, self.max_delay)
        if self.jitter:
            return_value = random.uniform(0.0, return_value)
        self.current_delay *= self.backoff
        return return_value


def _retry_func(
    func, sleep_time_generator, retries, catch_extra_error_codes, found_f, status_code_from_except_f, base_class
):
    counter = 0
    for sleep_time in sleep_time_generator:
        try:
            return func()
        except Exception as exc:  # pylint: disable=broad-except
            counter += 1
            if counter == retries:
                raise
            if base_class and not isinstance(exc, base_class):
                raise
            status_code = status_code_from_except_f(exc)
            if found_f(status_code, catch_extra_error_codes):
                time.sleep(sleep_time)
            else:
                raise


class CloudRetry:
    """
    The base class to be used by other cloud providers to provide a backoff/retry decorator based on status codes.
    """

    base_class = type(None)

    @staticmethod
    def status_code_from_exception(error):
        """
        Returns the Error 'code' from an exception.
        Args:
          error: The Exception from which the error code is to be extracted.
            error will be an instance of class.base_class.
        """
        raise NotImplementedError()

    @staticmethod
    def found(response_code, catch_extra_error_codes=None):
        def _is_iterable():
            try:
                iter(catch_extra_error_codes)
            except TypeError:
                # not iterable
                return False
            else:
                # iterable
                return True

        return _is_iterable() and response_code in catch_extra_error_codes

    @classmethod
    def base_decorator(cls, retries, found, status_code_from_exception, catch_extra_error_codes, sleep_time_generator):
        def retry_decorator(func):
            @functools.wraps(func)
            def _retry_wrapper(*args, **kwargs):
                partial_func = functools.partial(func, *args, **kwargs)
                return _retry_func(
                    func=partial_func,
                    sleep_time_generator=sleep_time_generator,
                    retries=retries,
                    catch_extra_error_codes=catch_extra_error_codes,
                    found_f=found,
                    status_code_from_except_f=status_code_from_exception,
                    base_class=cls.base_class,
                )

            return _retry_wrapper

        return retry_decorator

    @classmethod
    def exponential_backoff(cls, retries=10, delay=3, backoff=2, max_delay=60, catch_extra_error_codes=None):
        """Wrap a callable with retry behavior.
        Args:
            retries (int): Number of times to retry a failed request before giving up
                default=10
            delay (int or float): Initial delay between retries in seconds
                default=3
            backoff (int or float): backoff multiplier e.g. value of 2 will  double the delay each retry
                default=2
            max_delay (int or None): maximum amount of time to wait between retries.
                default=60
            catch_extra_error_codes: Additional error messages to catch, in addition to those which may be defined by a subclass of CloudRetry
                default=None
        Returns:
            Callable: A generator that calls the decorated function using an exponential backoff.
        """
        sleep_time_generator = BackoffIterator(delay=delay, backoff=backoff, max_delay=max_delay)
        return cls.base_decorator(
            retries=retries,
            found=cls.found,
            status_code_from_exception=cls.status_code_from_exception,
            catch_extra_error_codes=catch_extra_error_codes,
            sleep_time_generator=sleep_time_generator,
        )

    @classmethod
    def jittered_backoff(cls, retries=10, delay=3, backoff=2.0, max_delay=60, catch_extra_error_codes=None):
        """Wrap a callable with retry behavior.
        Args:
            retries (int): Number of times to retry a failed request before giving up
                default=10
            delay (int or float): Initial delay between retries in seconds
                default=3
            backoff (int or float): backoff multiplier e.g. value of 2 will  double the delay each retry
                default=2.0
            max_delay (int or None): maximum amount of time to wait between retries.
                default=60
            catch_extra_error_codes: Additional error messages to catch, in addition to those which may be defined by a subclass of CloudRetry
                default=None
        Returns:
            Callable: A generator that calls the decorated function using using a jittered backoff strategy.
        """
        sleep_time_generator = BackoffIterator(delay=delay, backoff=backoff, max_delay=max_delay, jitter=True)
        return cls.base_decorator(
            retries=retries,
            found=cls.found,
            status_code_from_exception=cls.status_code_from_exception,
            catch_extra_error_codes=catch_extra_error_codes,
            sleep_time_generator=sleep_time_generator,
        )



#!/usr/bin/python
#
# Copyright (c) 2023, Felix Fontein <felix@fontein.de>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: docker_image_tag

short_description: Tag Docker images with new names and/or tags

version_added: 3.6.0

description:
  - This module allows to tag Docker images with new names and/or tags.
extends_documentation_fragment:
  - community.docker._docker.api_documentation
  - community.docker._attributes
  - community.docker._attributes.actiongroup_docker

attributes:
  check_mode:
    support: full
  diff_mode:
    support: full
  idempotent:
    support: full

options:
  name:
    description:
      - 'Image name. Name format will be one of: C(name), C(repository/name), C(registry_server:port/name). When pushing or
        pulling an image the name can optionally include the tag by appending C(:tag_name).'
      - Note that image IDs (hashes) can also be used.
    type: str
    required: true
  tag:
    description:
      - Tag for the image name O(name) that is to be tagged.
      - If O(name)'s format is C(name:tag), then the tag value from O(name) will take precedence.
    type: str
    default: latest
  repository:
    description:
      - List of new image names to tag the image as.
      - Expects format C(repository:tag). If no tag is provided, will use the value of the O(tag) parameter if present, or
        V(latest).
    type: list
    elements: str
    required: true
  existing_images:
    description:
      - Defines the behavior if the image to be tagged already exists and is another image than the one identified by O(name)
        and O(tag).
      - If set to V(keep), the tagged image is kept.
      - If set to V(overwrite), the tagged image is overwritten by the specified one.
    type: str
    choices:
      - keep
      - overwrite
    default: overwrite

requirements:
  - "Docker API >= 1.25"

author:
  - Felix Fontein (@felixfontein)

seealso:
  - module: community.docker.docker_image_push
  - module: community.docker.docker_image_remove
"""

EXAMPLES = r"""
---
- name: Tag Python 3.12 image with two new names
  community.docker.docker_image_tag:
    name: python:3.12
    repository:
      - python-3:3.12
      - local-registry:5000/python-3/3.12:latest
"""

RETURN = r"""
image:
  description: Image inspection results for the affected image.
  returned: success
  type: dict
  sample: {}
tagged_images:
  description:
    - A list of images that got tagged.
  returned: success
  type: list
  elements: str
  sample:
    - python-3:3.12
"""

import traceback
import typing as t

from ansible.module_utils.common.text.formatters import human_to_bytes

from ansible_collections.community.docker.plugins.module_utils._api.errors import (
    DockerException,
)
from ansible_collections.community.docker.plugins.module_utils._api.utils.utils import (
    parse_repository_tag,
)
from ansible_collections.community.docker.plugins.module_utils._common_api import (
    AnsibleDockerClient,
    RequestException,
)
from ansible_collections.community.docker.plugins.module_utils._util import (
    DockerBaseClass,
    is_image_name_id,
    is_valid_tag,
)

if t.TYPE_CHECKING:
    from ansible.module_utils.basic import AnsibleModule


def convert_to_bytes(
    value: str | None,
    module: AnsibleModule,
    name: str,
    unlimited_value: int | None = None,
) -> int | None:
    if value is None:
        return value
    try:
        if unlimited_value is not None and value in ("unlimited", str(unlimited_value)):
            return unlimited_value
        return human_to_bytes(value)
    except ValueError as exc:
        module.fail_json(msg=f"Failed to convert {name} to bytes: {exc}")


def image_info(name: str, tag: str, image: dict[str, t.Any] | None) -> dict[str, t.Any]:
    result: dict[str, t.Any] = {"name": name, "tag": tag}
    if image:
        result["id"] = image["Id"]
    else:
        result["exists"] = False
    return result


class ImageTagger(DockerBaseClass):
    def __init__(self, client: AnsibleDockerClient) -> None:
        super().__init__()

        self.client = client
        parameters = self.client.module.params
        self.check_mode = self.client.check_mode

        self.name = parameters["name"]
        self.tag = parameters["tag"]
        if not is_valid_tag(self.tag, allow_empty=True):
            self.fail(f'"{self.tag}" is not a valid docker tag')

        # If name contains a tag, it takes precedence over tag parameter.
        if not is_image_name_id(self.name):
            repo, repo_tag = parse_repository_tag(self.name)
            if repo_tag:
                self.name = repo
                self.tag = repo_tag

        self.keep_existing_images = parameters["existing_images"] == "keep"

        # Make sure names in repository are valid images, and add tag if needed
        self.repositories = []
        for i, repository in enumerate(parameters["repository"]):
            if is_image_name_id(repository):
                self.fail(
                    f"repository[{i + 1}] must not be an image ID; got: {repository}"
                )
            repo, repo_tag = parse_repository_tag(repository)
            if not repo_tag:
                repo_tag = parameters["tag"]
            elif not is_valid_tag(repo_tag, allow_empty=False):
                self.fail(
                    f"repository[{i + 1}] must not have a digest; got: {repository}"
                )
            self.repositories.append((repo, repo_tag))

    def fail(self, msg: str) -> t.NoReturn:
        self.client.fail(msg)

    def tag_image(
        self, image: dict[str, t.Any], name: str, tag: str
    ) -> tuple[bool, str, dict[str, t.Any] | None]:
        tagged_image = self.client.find_image(name=name, tag=tag)
        if tagged_image:
            # Idempotency checks
            if tagged_image["Id"] == image["Id"]:
                return (
                    False,
                    f"target image already exists ({tagged_image['Id']}) and is as expected",
                    tagged_image,
                )
            if self.keep_existing_images:
                return (
                    False,
                    f"target image already exists ({tagged_image['Id']}) and is not as expected, but kept",
                    tagged_image,
                )
            msg = f"target image existed ({tagged_image['Id']}) and was not as expected"
        else:
            msg = "target image did not exist"

        if not self.check_mode:
            try:
                params = {
                    "tag": tag,
                    "repo": name,
                    "force": True,
                }
                res = self.client._post(
                    self.client._url("/images/{0}/tag", image["Id"]), params=params
                )
                self.client._raise_for_status(res)
                if res.status_code != 201:
                    raise RuntimeError("Tag operation failed.")
            except Exception as exc:  # pylint: disable=broad-exception-caught
                self.fail(f"Error: failed to tag image as {name}:{tag} - {exc}")

        return True, msg, tagged_image

    def tag_images(self) -> dict[str, t.Any]:
        if is_image_name_id(self.name):
            image = self.client.find_image_by_id(self.name, accept_missing_image=False)
        else:
            image = self.client.find_image(name=self.name, tag=self.tag)
            if not image:
                self.fail(f"Cannot find image {self.name}:{self.tag}")
        assert image is not None

        before: list[dict[str, t.Any]] = []
        after: list[dict[str, t.Any]] = []
        tagged_images: list[str] = []
        actions: list[str] = []
        results: dict[str, t.Any] = {
            "changed": False,
            "actions": actions,
            "image": image,
            "tagged_images": tagged_images,
            "diff": {"before": {"images": before}, "after": {"images": after}},
        }
        for repository, tag in self.repositories:
            tagged, msg, old_image = self.tag_image(image, repository, tag)
            before.append(image_info(repository, tag, old_image))
            after.append(image_info(repository, tag, image if tagged else old_image))
            if tagged:
                results["changed"] = True
                actions.append(
                    f"Tagged image {image['Id']} as {repository}:{tag}: {msg}"
                )
                tagged_images.append(f"{repository}:{tag}")
            else:
                actions.append(
                    f"Not tagged image {image['Id']} as {repository}:{tag}: {msg}"
                )

        return results


def main() -> None:
    argument_spec = {
        "name": {"type": "str", "required": True},
        "tag": {"type": "str", "default": "latest"},
        "repository": {"type": "list", "elements": "str", "required": True},
        "existing_images": {
            "type": "str",
            "choices": ["keep", "overwrite"],
            "default": "overwrite",
        },
    }

    client = AnsibleDockerClient(
        argument_spec=argument_spec,
        supports_check_mode=True,
    )

    try:
        results = ImageTagger(client).tag_images()
        client.module.exit_json(**results)
    except DockerException as e:
        client.fail(
            f"An unexpected Docker error occurred: {e}",
            exception=traceback.format_exc(),
        )
    except RequestException as e:
        client.fail(
            f"An unexpected requests error occurred when trying to talk to the Docker daemon: {e}",
            exception=traceback.format_exc(),
        )


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: networks_pii_sm_owners_for_key_info
short_description: Information module for networks _pii _sm _owners _for _key
description:
  - Get all networks _pii _sm _owners _for _key. - > Given a piece of Personally Identifiable
    Information PII , return the Systems Manager owner IDs associated with that identifier.
    These owner IDs can be used with the Systems Manager API endpoints to retrieve
    owner details. Exactly one identifier will be accepted.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module_info
author: Francisco Munoz (@fmunoz)
options:
  headers:
    description: Additional headers.
    type: dict
  networkId:
    description:
      - NetworkId path parameter. Network ID.
    type: str
  username:
    description:
      - Username query parameter. The username of a Systems Manager user.
    type: str
  email:
    description:
      - Email query parameter. The email of a network user account or a Systems Manager
        device.
    type: str
  mac:
    description:
      - Mac query parameter. The MAC of a network client device or a Systems Manager
        device.
    type: str
  serial:
    description:
      - Serial query parameter. The serial of a Systems Manager device.
    type: str
  imei:
    description:
      - Imei query parameter. The IMEI of a Systems Manager device.
    type: str
  bluetoothMac:
    description:
      - BluetoothMac query parameter. The MAC of a Bluetooth client.
    type: str
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for networks getNetworkPiiSmOwnersForKey
    description: Complete reference of the getNetworkPiiSmOwnersForKey API.
    link: https://developer.cisco.com/meraki/api-v1/#!get-network-pii-sm-owners-for-key
notes:
  - SDK Method used are
    networks.Networks.get_network_pii_sm_owners_for_key,
  - Paths used are
    get /networks/{networkId}/pii/smOwnersForKey,
"""

EXAMPLES = r"""
- name: Get all networks _pii _sm _owners _for _key
  cisco.meraki.networks_pii_sm_owners_for_key_info:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    username: string
    email: string
    mac: string
    serial: string
    imei: string
    bluetoothMac: string
    networkId: string
  register: result
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample:
  - {}
"""



#!/usr/bin/python
# Copyright (c) 2016 Michael Gruener <michael.gruener@chaosmoon.net>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: acme_account
author: "Felix Fontein (@felixfontein)"
short_description: Create, modify or delete ACME accounts
description:
  - Allows to create, modify or delete accounts with a CA supporting the L(ACME protocol,https://tools.ietf.org/html/rfc8555),
    such as L(Let's Encrypt,https://letsencrypt.org/).
  - This module only works with the ACME v2 protocol.
notes:
  - The M(community.crypto.acme_certificate) module also allows to do basic account management. When using both modules, it
    is recommended to disable account management for M(community.crypto.acme_certificate). For that, use the
    O(community.crypto.acme_certificate#module:modify_account) option of M(community.crypto.acme_certificate).
seealso:
  - name: Automatic Certificate Management Environment (ACME)
    description: The specification of the ACME protocol (RFC 8555).
    link: https://tools.ietf.org/html/rfc8555
  - module: community.crypto.acme_account_info
    description: Retrieves facts about an ACME account.
  - module: community.crypto.openssl_privatekey
    description: Can be used to create a private account key.
  - module: community.crypto.openssl_privatekey_pipe
    description: Can be used to create a private account key without writing it to disk.
  - module: community.crypto.acme_inspect
    description: Allows to debug problems.
extends_documentation_fragment:
  - community.crypto._acme.basic
  - community.crypto._acme.account
  - community.crypto._attributes
  - community.crypto._attributes.actiongroup_acme
attributes:
  check_mode:
    support: full
  diff_mode:
    support: full
  idempotent:
    support: partial
    details:
      - If O(state=changed_key) is used, the module is not idempotent.
options:
  state:
    description:
      - The state of the account, to be identified by its account key.
      - If the state is V(absent), the account will either not exist or be deactivated.
      - If the state is V(changed_key), the account must exist. The account key will be changed; no other information will
        be touched.
    type: str
    required: true
    choices:
      - present
      - absent
      - changed_key
  allow_creation:
    description:
      - Whether account creation is allowed (when state is V(present)).
    type: bool
    default: true
  contact:
    description:
      - A list of contact URLs.
      - Email addresses must be prefixed with C(mailto:).
      - See U(https://tools.ietf.org/html/rfc8555#section-7.3) for what is allowed.
      - Must be specified when state is V(present). Will be ignored if state is V(absent) or V(changed_key).
    type: list
    elements: str
    default: []
  terms_agreed:
    description:
      - Boolean indicating whether you agree to the terms of service document.
      - ACME servers can require this to be V(true).
    type: bool
    default: false
  new_account_key_src:
    description:
      - Path to a file containing the ACME account RSA or Elliptic Curve key to change to.
      - Same restrictions apply as to O(account_key_src).
      - Mutually exclusive with O(new_account_key_content).
      - Required if O(new_account_key_content) is not used and O(state) is V(changed_key).
    type: path
  new_account_key_content:
    description:
      - Content of the ACME account RSA or Elliptic Curve key to change to.
      - Same restrictions apply as to O(account_key_content).
      - Mutually exclusive with O(new_account_key_src).
      - Required if O(new_account_key_src) is not used and O(state) is V(changed_key).
    type: str
  new_account_key_passphrase:
    description:
      - Passphrase to use to decode the new account key.
      - B(Note:) this is not supported by the C(openssl) backend, only by the C(cryptography) backend.
    type: str
    version_added: 1.6.0
  external_account_binding:
    description:
      - Allows to provide external account binding data during account creation.
      - This is used by CAs like Sectigo, HARICA, or ZeroSSL to bind a new ACME account to an existing CA-specific account,
        to be able to properly identify a customer.
      - Only used when creating a new account. Can not be specified for ACME v1.
    type: dict
    suboptions:
      kid:
        description:
          - The key identifier provided by the CA.
        type: str
        required: true
      alg:
        description:
          - The MAC algorithm provided by the CA.
          - If not specified by the CA, this is probably V(HS256).
        type: str
        required: true
        choices: [HS256, HS384, HS512]
      key:
        description:
          - Base64 URL encoded value of the MAC key provided by the CA.
          - Padding (V(=) symbols at the end) can be omitted.
        type: str
        required: true
    version_added: 1.1.0
"""

EXAMPLES = r"""
---
- name: Make sure account exists and has given contacts. We agree to TOS.
  community.crypto.acme_account:
    account_key_src: /etc/pki/cert/private/account.key
    state: present
    terms_agreed: true
    contact:
      - mailto:me@example.com
      - mailto:myself@example.org

- name: Make sure account has given email address. Do not create account if it does not exist
  community.crypto.acme_account:
    account_key_src: /etc/pki/cert/private/account.key
    state: present
    allow_creation: false
    contact:
      - mailto:me@example.com

- name: Change account's key to the one stored in the variable new_account_key
  community.crypto.acme_account:
    account_key_src: /etc/pki/cert/private/account.key
    new_account_key_content: '{{ new_account_key }}'
    state: changed_key

- name: Delete account (we have to use the new key)
  community.crypto.acme_account:
    account_key_content: '{{ new_account_key }}'
    state: absent
"""

RETURN = r"""
account_uri:
  description: ACME account URI, or None if account does not exist.
  returned: always
  type: str
"""

import base64
import typing as t

from ansible_collections.community.crypto.plugins.module_utils._acme.account import (
    ACMEAccount,
)
from ansible_collections.community.crypto.plugins.module_utils._acme.acme import (
    ACMEClient,
    create_backend,
    create_default_argspec,
)
from ansible_collections.community.crypto.plugins.module_utils._acme.errors import (
    KeyParsingError,
    ModuleFailException,
)


def main() -> t.NoReturn:
    argument_spec = create_default_argspec()
    argument_spec.update_argspec(
        terms_agreed={"type": "bool", "default": False},
        state={
            "type": "str",
            "required": True,
            "choices": ["absent", "present", "changed_key"],
        },
        allow_creation={"type": "bool", "default": True},
        contact={"type": "list", "elements": "str", "default": []},
        new_account_key_src={"type": "path"},
        new_account_key_content={"type": "str", "no_log": True},
        new_account_key_passphrase={"type": "str", "no_log": True},
        external_account_binding={
            "type": "dict",
            "options": {
                "kid": {"type": "str", "required": True},
                "alg": {
                    "type": "str",
                    "required": True,
                    "choices": ["HS256", "HS384", "HS512"],
                },
                "key": {"type": "str", "required": True, "no_log": True},
            },
        },
    )
    argument_spec.update(
        mutually_exclusive=[("new_account_key_src", "new_account_key_content")],
        required_if=[
            # Make sure that for state == changed_key, one of
            # new_account_key_src and new_account_key_content are specified
            (
                "state",
                "changed_key",
                ["new_account_key_src", "new_account_key_content"],
                True,
            ),
        ],
    )
    module = argument_spec.create_ansible_module(supports_check_mode=True)
    backend = create_backend(module, needs_acme_v2=True)

    if module.params["external_account_binding"]:
        # Make sure padding is there
        key: str = module.params["external_account_binding"]["key"]
        if len(key) % 4 != 0:
            key = key + ("=" * (4 - (len(key) % 4)))
        # Make sure key is Base64 encoded
        try:
            base64.urlsafe_b64decode(key)
        except Exception as e:
            module.fail_json(
                msg=f"Key for external_account_binding must be Base64 URL encoded ({e})"
            )
        module.params["external_account_binding"]["key"] = key

    try:
        client = ACMEClient(module=module, backend=backend)
        account = ACMEAccount(client=client)
        changed = False
        state: t.Literal["present", "absent", "changed_key"] = module.params["state"]
        diff_before: dict[str, t.Any] = {}
        diff_after: dict[str, t.Any] = {}
        if state == "absent":
            created, account_data = account.setup_account(allow_creation=False)
            if account_data:
                diff_before = dict(account_data)
                if client.account_key_data:
                    diff_before["public_account_key"] = client.account_key_data["jwk"]
            if created:
                raise AssertionError("Unwanted account creation")  # pragma: no cover
            if account_data is not None:
                # Account is not yet deactivated
                if not module.check_mode:
                    # Deactivate it
                    deactivate_payload = {"status": "deactivated"}
                    result, _info = client.send_signed_request(
                        t.cast(str, client.account_uri),
                        deactivate_payload,
                        error_msg="Failed to deactivate account",
                        expected_status_codes=[200],
                    )
                changed = True
        elif state == "present":
            allow_creation = module.params.get("allow_creation")
            contact = [str(v) for v in module.params.get("contact")]
            terms_agreed = module.params.get("terms_agreed")
            external_account_binding = module.params.get("external_account_binding")
            created, account_data = account.setup_account(
                contact=contact,
                terms_agreed=terms_agreed,
                allow_creation=allow_creation,
                external_account_binding=external_account_binding,
            )
            if account_data is None:
                raise ModuleFailException(
                    msg="Account does not exist or is deactivated."
                )
            if created:
                diff_before = {}
            else:
                diff_before = dict(account_data)
                if client.account_key_data:
                    diff_before["public_account_key"] = client.account_key_data["jwk"]
            updated = False
            if not created:
                updated, account_data = account.update_account(
                    account_data=account_data, contact=contact
                )
            changed = created or updated
            diff_after = dict(account_data)
            if client.account_key_data:
                diff_after["public_account_key"] = client.account_key_data["jwk"]
        elif state == "changed_key":
            # Parse new account key
            try:
                new_key_data = client.parse_key(
                    key_file=module.params.get("new_account_key_src"),
                    key_content=module.params.get("new_account_key_content"),
                    passphrase=module.params.get("new_account_key_passphrase"),
                )
            except KeyParsingError as e:
                raise ModuleFailException(
                    f"Error while parsing new account key: {e.msg}"
                ) from e
            # Verify that the account exists and has not been deactivated
            created, account_data = account.setup_account(allow_creation=False)
            if created:
                raise AssertionError("Unwanted account creation")  # pragma: no cover
            if account_data is None:
                raise ModuleFailException(
                    msg="Account does not exist or is deactivated."
                )
            diff_before = dict(account_data)
            if client.account_key_data:
                diff_before["public_account_key"] = client.account_key_data["jwk"]
            # Now we can start the account key rollover
            if not module.check_mode:
                # Compose inner signed message
                # https://tools.ietf.org/html/rfc8555#section-7.3.5
                url = client.directory["keyChange"]
                protected = {
                    "alg": new_key_data["alg"],
                    "jwk": new_key_data["jwk"],
                    "url": url,
                }
                change_key_payload = {
                    "account": client.account_uri,
                    "newKey": new_key_data["jwk"],  # specified in draft 12 and older
                    "oldKey": client.account_jwk,  # specified in draft 13 and newer
                }
                data = client.sign_request(
                    protected=protected,
                    payload=change_key_payload,
                    key_data=new_key_data,
                )
                # Send request and verify result
                result, _info = client.send_signed_request(
                    url,
                    data,
                    error_msg="Failed to rollover account key",
                    expected_status_codes=[200],
                )
                if module._diff:  # pylint: disable=protected-access
                    client.account_key_data = new_key_data
                    if client.account_jws_header:
                        client.account_jws_header["alg"] = new_key_data["alg"]
                    diff_after = account.get_account_data() or {}
            elif module._diff:  # pylint: disable=protected-access
                # Kind of fake diff_after
                diff_after = dict(diff_before)
            diff_after["public_account_key"] = new_key_data["jwk"]
            changed = True
        result = {
            "changed": changed,
            "account_uri": client.account_uri,
        }
        if module._diff:  # pylint: disable=protected-access
            result["diff"] = {
                "before": diff_before,
                "after": diff_after,
            }
        module.exit_json(**result)
    except ModuleFailException as e:
        e.do_fail(module=module)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: organizations_wireless_devices_channel_utilization_by_network_info
short_description: Information module for organizations _wireless _devices _channel
  _utilization _by _network
description:
  - Get all organizations _wireless _devices _channel _utilization _by _network.
  - Get average channel utilization across all bands for all networks in the organization.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module_info
  - cisco.meraki.module_info_pagination
author: Francisco Munoz (@fmunoz)
options:
  headers:
    description: Additional headers.
    type: dict
  organizationId:
    description:
      - OrganizationId path parameter. Organization ID.
    type: str
  networkIds:
    description:
      - NetworkIds query parameter. Filter results by network.
    elements: str
    type: list
  serials:
    description:
      - Serials query parameter. Filter results by device.
    elements: str
    type: list
  perPage:
    description:
      - PerPage query parameter. The number of entries per page returned. Acceptable
        range is 3 - 1000. Default is 1000.
    type: int
  startingAfter:
    description:
      - >
        StartingAfter query parameter. A token used by the server to indicate the
        start of the page. Often this is a timestamp or an ID but it is not limited
        to those. This parameter should not be defined by client applications. The
        link for the first, last, prev, or next page in the HTTP Link header should
        define it.
    type: str
  endingBefore:
    description:
      - >
        EndingBefore query parameter. A token used by the server to indicate the end
        of the page. Often this is a timestamp or an ID but it is not limited to those.
        This parameter should not be defined by client applications. The link for
        the first, last, prev, or next page in the HTTP Link header should define
        it.
    type: str
  t0:
    description:
      - T0 query parameter. The beginning of the timespan for the data. The maximum
        lookback period is 90 days from today.
    type: str
  t1:
    description:
      - T1 query parameter. The end of the timespan for the data. T1 can be a maximum
        of 90 days after t0.
    type: str
  timespan:
    description:
      - >
        Timespan query parameter. The timespan for which the information will be fetched.
        If specifying timespan, do not specify parameters t0 and t1. The value must
        be in seconds and be less than or equal to 90 days. The default is 7 days.
    type: float
  interval:
    description:
      - >
        Interval query parameter. The time interval in seconds for returned data.
        The valid intervals are 300, 600, 3600, 7200, 14400, 21600. The default is
        3600.
    type: int
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for wireless getOrganizationWirelessDevicesChannelUtilizationByNetwork
    description: Complete reference of the getOrganizationWirelessDevicesChannelUtilizationByNetwork
      API.
    link: https://developer.cisco.com/meraki/api-v1/#!get-organization-wireless-devices-channel-utilization-by-network
notes:
  - SDK Method used are
    wireless.Wireless.get_organization_wireless_devices_channel_utilization_by_network,
  - Paths used are
    get /organizations/{organizationId}/wireless/devices/channelUtilization/byNetwork,
"""

EXAMPLES = r"""
- name: Get all organizations _wireless _devices _channel _utilization _by _network
  cisco.meraki.organizations_wireless_devices_channel_utilization_by_network_info:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    networkIds: []
    serials: []
    perPage: 0
    startingAfter: string
    endingBefore: string
    t0: string
    t1: string
    timespan: 0
    interval: 0
    organizationId: string
    total_pages: -1
    direction: next
  register: result
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: list
  elements: dict
  sample: >
    [
      {
        "network": {
          "id": "string"
        },
        "byBand": [
          {
            "band": "string",
            "wifi": {
              "percentage": 0
            },
            "nonWifi": {
              "percentage": 0
            },
            "total": {
              "percentage": 0
            }
          }
        ]
      }
    ]
"""



#!/usr/bin/python
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

DOCUMENTATION = '''
---
module: azure_rm_ml_online_endpoint
version_added: "3.17.0"
short_description: Create, Delete or regenerate keys for an Azure Machine Learning Online Endpoint
description:
    - Create, Delete or regenerate keys for an Azure Machine Learning Online Endpoint.
options:
    name:
        description:
            - Name of the Azure ML online_endpoint.
        type: str
        required: true
    resource_group:
        description:
            - Name of resource group.
        required: true
        type: str
    ml_workspace:
        description:
            - Name of the Azure ML workspace.
        required: true
        type: str
    resource_definition:
        description:
            - Online Endpoint definition in YAML
        type: str
        required: false
    description:
        description:
            - Description of the online_endpoint.
        type: str
    local:
        description:
            - Update local endpoint.
        type: bool
        default: false
    auth_mode:
        description:
            - Authentication method for the endpoint.
        default: key
        type: str
        choices:
            - key
            - aml_token
    mirror_traffic:
        description:
            - Key and Numeric value to Direct a duplicate percentage
              of live traffic to a train a deployment.
        required: false
        type: dict
    traffic:
        description:
            - Key and Numeric value for traffic settings of this endpoint.
        required: false
        type: dict
    key_type:
        description:
            - The type of key to regenerate.
        type: str
        default: primary
        choices:
            - primary
            - secondary
    state:
        description:
            - State of the Online Endpoint. Use C(present) to create
              or update. C(regenerate_keys) to regenerate keys.
        default: present
        type: str
        choices:
            - present
            - absent
            - regenerate_keys

extends_documentation_fragment:
    - azure.azcollection.azure
    - azure.azcollection.azure_tags

author:
    - Bill Peck (@p3ck)
'''

EXAMPLES = '''
- name: Create ML Online Endpoint
  azure.azcollection.azure_rm_ml_online_endpoint:
    name: onlineendpointxxxxxxxxxx
    resource_group: xxxxx-ml-online_endpoint
    ml_workspace: workspace-xxxxxxxxxx
    resource_definition: |
'''

RETURN = '''
changed:
    description:
        - Whether the resource is changed.
    returned: always
    type: bool
    sample: false
ml_online_endpoint:
    description:
        - Online Endpoint that was just created or updated.
    returned: always
    type: dict
    sample: {
      "auth_mode": "key",
      "id": "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/xxxxx-ml-online_endpoint/providers/Microsoft.MachineLearningServices/workspaces/workspace-xxxxxxxxxx/onlineEndpoints/endpoint-command-xxxxxxxxxx",
      "identity": {
          "principal_id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
          "tenant_id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
          "type": "system_assigned"
      },
      "kind": "Managed",
      "location": "eastus",
      "mirror_traffic": {},
      "name": "endpoint-command-xxxxxxxxxx",
      "openapi_uri": "https://endpoint-command-xxxxxxxxxx.eastus.inference.ml.azure.com/swagger.json",
      "properties": {
          "AzureAsyncOperationUri": "https://management.azure.com/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/providers/Microsoft.MachineLearningServices/locations/eastus/mfeOperationsStatus/oeidp:xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx:xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx?api-version=2022-02-01-preview",
          "azureml.onlineendpointid": "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourcegroups/xxxxx-ml-online_endpoint/providers/microsoft.machinelearningservices/workspaces/workspace-xxxxxxxxxx/onlineendpoints/endpoint-command-xxxxxxxxxx",
          "createdAt": "2026-04-09T15:04:48.831114+0000",
          "createdBy": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
          "lastModifiedAt": "2026-04-09T15:04:48.831114+0000"
      },
      "provisioning_state": "Succeeded",
      "public_network_access": "enabled",
      "scoring_uri": "https://endpoint-command-xxxxxxxxxx.eastus.inference.ml.azure.com/score",
      "tags": {},
      "traffic": {}
    }
'''  # NOQA


try:
    import io
    from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common_ml import MLClientCommon
    from azure.core.exceptions import ResourceNotFoundError
    from azure.ai.ml.entities._load_functions import load_online_endpoint
    from azure.ai.ml.constants._endpoint import EndpointKeyType
except ImportError:
    # This is handled in azure_rm_common
    pass


class AzureRMMLOnlineEndpoint(MLClientCommon):

    UPDATE_OVERRIDES = ['description',
                        'tags',
                        'mirror_traffic',
                        'traffic']

    PARAM_OVERRIDES = UPDATE_OVERRIDES + ['name',
                                          'auth_mode']

    def __init__(self):

        self.module_arg_spec = dict(
            name=dict(
                type='str',
                required=True,
            ),
            resource_definition=dict(
                type='str',
                required=False,
            ),
            resource_group=dict(
                type='str',
                required=True,
            ),
            ml_workspace=dict(
                type='str',
                required=True,
            ),
            local=dict(
                type='bool',
                default=False,
            ),
            description=dict(
                type='str',
                required=False,
            ),
            auth_mode=dict(
                type='str',
                default='key',
                choices=['key', 'aml_token'],
            ),
            mirror_traffic=dict(
                type='dict',
                required=False,
            ),
            traffic=dict(
                type='dict',
                required=False,
            ),
            key_type=dict(
                type='str',
                default='primary',
                choices=['primary', 'secondary'],
            ),
            state=dict(
                type='str',
                default='present',
                choices=['present', 'absent', 'regenerate_keys'],
            ),
        )

        self._client = None
        self.ml_registry = None
        self.state = None

        self.results = dict(
            changed=False,
            compare=[],
            ml_online_endpoint={}
        )

        required_if = [
            ('state', 'present', ['resource_definition'])
        ]

        super(AzureRMMLOnlineEndpoint, self).__init__(self.module_arg_spec,
                                                      supports_check_mode=True,
                                                      required_if=required_if,
                                                      )

    def exec_module(self, **kwargs):

        for key in list(self.module_arg_spec.keys()) + ['tags']:
            setattr(self, key, kwargs[key])

        changed = False

        ml_online_endpoint_info = self.get(name=self.name,
                                           local=self.local)

        if self.state == 'present':
            if self.resource_definition:
                resource_definition = io.StringIO(self.resource_definition)

            if ml_online_endpoint_info:
                # Update
                params_override = self.update_params(kwargs,
                                                     self.UPDATE_OVERRIDES,
                                                     ml_online_endpoint_info)

                # Generate ml_online_endpoint based on ansible args passed in.
                ml_online_endpoint = load_online_endpoint(resource_definition,
                                                          params_override=params_override)
                ml_online_endpoint_info = self.entity_to_dict(ml_online_endpoint_info)
                changed = not self.default_compare({},
                                                   self.entity_to_dict(ml_online_endpoint),
                                                   ml_online_endpoint_info,
                                                   '',
                                                   self.results)
                if changed and not self.check_mode:
                    response = self.client.begin_create_or_update(ml_online_endpoint,
                                                                  local=self.local)
                    ml_online_endpoint_object = self.get_poller_result(response)
                    ml_online_endpoint_info = self.entity_to_dict(ml_online_endpoint_object)
            else:
                # Create
                params_override = self.update_params(kwargs, self.PARAM_OVERRIDES)

                # Generate ml_online_endpoint based on ansible args passed in.
                ml_online_endpoint = load_online_endpoint(source=resource_definition,
                                                          params_override=params_override)

                changed = True
                if not self.check_mode:
                    response = self.client.begin_create_or_update(ml_online_endpoint,
                                                                  local=self.local)
                    ml_online_endpoint_object = self.get_poller_result(response)
                    ml_online_endpoint_info = self.entity_to_dict(ml_online_endpoint_object)
        elif self.state == 'absent':
            if ml_online_endpoint_info:
                # Delete
                changed = True
                if not self.check_mode:
                    response = self.client.online_endpoints.begin_delete(name=self.name,
                                                                         local=self.local)
                    ml_online_endpoint_info = self.get_poller_result(response)
        elif self.state == 'regenerate_keys':
            if ml_online_endpoint_info:
                changed = True
                if not self.check_mode:
                    key_type = self.get_key_type(self.key_type)
                    self.client.online_endpoints.begin_regenerate_keys(name=self.name,
                                                                       key_type=key_type)
                ml_online_endpoint_info = self.entity_to_dict(ml_online_endpoint_info)
            else:
                # Failure.  can't regenerate keys for entity that doesn't exist.
                self.fail("Online Endpoint doesn't exist")

        self.results['ml_online_endpoint'] = ml_online_endpoint_info
        self.results['changed'] = changed
        return self.results

    def update_params(self, kwargs, param_overrides, ml_online_endpoint=None):
        params_override = []

        # If ml_online_endpoint is defined use those values as a base.
        # Update can't change some of these values but we need
        # to keep the same values as before the update.
        if ml_online_endpoint:
            items = self.entity_to_dict(ml_online_endpoint).items()
            params_override = [{k: v} for k, v in items]

        # Update values based on what param_overrides says we can update.
        for key in list(self.module_arg_spec.keys()):
            if kwargs[key] is not None and key in param_overrides:
                params_override.append({key: kwargs[key]})
        return params_override

    def get(self, name, local=None, as_dict=False):
        try:
            # Attempt to retrieve ml_online_endpoint based on name
            ml_online_endpoint_info = self.client.online_endpoints.get(name=name,
                                                                       local=local)
            if as_dict:
                ml_online_endpoint_info = self.entity_to_dict(ml_online_endpoint_info)
        except ResourceNotFoundError:
            ml_online_endpoint_info = None
        return ml_online_endpoint_info

    def get_key_type(self, key_type):
        key_types = dict(primary=EndpointKeyType.PRIMARY_KEY_TYPE,
                         secondary=EndpointKeyType.SECONDARY_KEY_TYPE)
        return key_types.get(key_type, EndpointKeyType.PRIMARY_KEY_TYPE)


def main():
    AzureRMMLOnlineEndpoint()


if __name__ == '__main__':
    main()



# Copyright (c) Stanislav Meduna (@numo68)
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

import typing as t
from json import loads

from ansible.errors import AnsibleFilterError

if t.TYPE_CHECKING:
    from collections.abc import Callable

JSONPATCH_IMPORT_ERROR: ImportError | None
try:
    import jsonpatch
except ImportError as exc:
    HAS_LIB = False
    JSONPATCH_IMPORT_ERROR = exc
else:
    HAS_LIB = True
    JSONPATCH_IMPORT_ERROR = None

OPERATIONS_AVAILABLE = ["add", "copy", "move", "remove", "replace", "test"]
OPERATIONS_NEEDING_FROM = ["copy", "move"]
OPERATIONS_NEEDING_VALUE = ["add", "replace", "test"]


class FilterModule:
    """Filter plugin."""

    def check_json_object(self, filter_name: str, object_name: str, inp: t.Any):
        if isinstance(inp, (str, bytes, bytearray)):
            try:
                return loads(inp)
            except Exception as e:
                raise AnsibleFilterError(f"{filter_name}: could not decode JSON from {object_name}: {e}") from e

        if not isinstance(inp, (list, dict)):
            raise AnsibleFilterError(f"{filter_name}: {object_name} is not dictionary, list or string")

        return inp

    def check_patch_arguments(self, filter_name: str, args: dict):
        if "op" not in args or not isinstance(args["op"], str):
            raise AnsibleFilterError(f"{filter_name}: 'op' argument is not a string")

        if args["op"] not in OPERATIONS_AVAILABLE:
            raise AnsibleFilterError(f"{filter_name}: unsupported 'op' argument: {args['op']}")

        if "path" not in args or not isinstance(args["path"], str):
            raise AnsibleFilterError(f"{filter_name}: 'path' argument is not a string")

        if args["op"] in OPERATIONS_NEEDING_FROM:
            if "from" not in args:
                raise AnsibleFilterError(f"{filter_name}: 'from' argument missing for '{args['op']}' operation")
            if not isinstance(args["from"], str):
                raise AnsibleFilterError(f"{filter_name}: 'from' argument is not a string")

    def json_patch(
        self,
        inp: str | list | dict | bytes | bytearray,
        op: str,
        path: str,
        value: t.Any = None,
        **kwargs: dict,
    ) -> t.Any:
        if not HAS_LIB:
            raise AnsibleFilterError(
                "You need to install 'jsonpatch' package prior to running 'json_patch' filter"
            ) from JSONPATCH_IMPORT_ERROR

        args: dict[str, t.Any] = {"op": op, "path": path}
        from_arg = kwargs.pop("from", None)
        fail_test = kwargs.pop("fail_test", False)

        if kwargs:
            raise AnsibleFilterError(f"json_patch: unexpected keywords arguments: {', '.join(sorted(kwargs))}")

        if not isinstance(fail_test, bool):
            raise AnsibleFilterError("json_patch: 'fail_test' argument is not a bool")

        if op in OPERATIONS_NEEDING_VALUE:
            args["value"] = value
        if op in OPERATIONS_NEEDING_FROM and from_arg is not None:
            args["from"] = from_arg

        inp = self.check_json_object("json_patch", "input", inp)
        self.check_patch_arguments("json_patch", args)

        result = None

        try:
            result = jsonpatch.apply_patch(inp, [args])
        except jsonpatch.JsonPatchTestFailed as e:
            if fail_test:
                raise AnsibleFilterError(f"json_patch: test operation failed: {e}") from e
            else:
                pass
        except Exception as e:
            raise AnsibleFilterError(f"json_patch: patch failed: {e}") from e

        return result

    def json_patch_recipe(
        self,
        inp: str | list | dict | bytes | bytearray,
        operations: list,
        /,
        fail_test: bool = False,
    ) -> t.Any:
        if not HAS_LIB:
            raise AnsibleFilterError(
                "You need to install 'jsonpatch' package prior to running 'json_patch_recipe' filter"
            ) from JSONPATCH_IMPORT_ERROR

        if not isinstance(operations, list):
            raise AnsibleFilterError("json_patch_recipe: 'operations' needs to be a list")

        if not isinstance(fail_test, bool):
            raise AnsibleFilterError("json_patch: 'fail_test' argument is not a bool")

        result = None

        inp = self.check_json_object("json_patch_recipe", "input", inp)
        for args in operations:
            self.check_patch_arguments("json_patch_recipe", args)

        try:
            result = jsonpatch.apply_patch(inp, operations)
        except jsonpatch.JsonPatchTestFailed as e:
            if fail_test:
                raise AnsibleFilterError(f"json_patch_recipe: test operation failed: {e}") from e
            else:
                pass
        except Exception as e:
            raise AnsibleFilterError(f"json_patch_recipe: patch failed: {e}") from e

        return result

    def json_diff(
        self,
        inp: str | list | dict | bytes | bytearray,
        target: str | list | dict | bytes | bytearray,
    ) -> list:
        if not HAS_LIB:
            raise AnsibleFilterError(
                "You need to install 'jsonpatch' package prior to running 'json_diff' filter"
            ) from JSONPATCH_IMPORT_ERROR

        inp = self.check_json_object("json_diff", "input", inp)
        target = self.check_json_object("json_diff", "target", target)

        try:
            result = list(jsonpatch.make_patch(inp, target))
        except Exception as e:
            raise AnsibleFilterError(f"JSON diff failed: {e}") from e

        return result

    def filters(self) -> dict[str, Callable[..., t.Any]]:
        """Map filter plugin names to their functions.

        Returns:
            dict: The filter plugin functions.
        """
        return {
            "json_patch": self.json_patch,
            "json_patch_recipe": self.json_patch_recipe,
            "json_diff": self.json_diff,
        }



#!/usr/bin/python
#
# Copyright (c) 2018 Yunge Zhu, <yungez@microsoft.com>
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_webappslot
version_added: "0.1.2"
short_description: Manage Azure Web App slot
description:
    - Create, update and delete Azure Web App slot.

options:
    resource_group:
        description:
            - Name of the resource group to which the resource belongs.
        required: True
        type: str
    name:
        description:
            - Unique name of the deployment slot to create or update.
        required: True
        type: str
    webapp_name:
        description:
            - Web app name which this deployment slot belongs to.
        required: True
        type: str
    location:
        description:
            - Resource location. If not set, location from the resource group will be used as default.
        type: str
    configuration_source:
        description:
            - Source slot to clone configurations from when creating slot. Use webapp's name to refer to the production slot.
        type: str
    auto_swap_slot_name:
        description:
            - Used to configure target slot name to auto swap, or disable auto swap.
            - Set it target slot name to auto swap.
            - Set it to False to disable auto slot swap.
        type: raw
    swap:
        description:
            - Swap deployment slots of a web app.
        type: dict
        suboptions:
            action:
                description:
                    - Swap types.
                    - C(preview) is to apply target slot settings on source slot first.
                    - C(swap) is to complete swapping.
                    - C(reset) is to reset the swap.
                type: str
                choices:
                    - preview
                    - swap
                    - reset
                default: preview
            target_slot:
                description:
                    - Name of target slot to swap. If set to None, then swap with production slot.
                type: str
            preserve_vnet:
                description:
                    - C(True) to preserve virtual network to the slot during swap. Otherwise C(False).
                type: bool
                default: True
    frameworks:
        description:
            - Set of run time framework settings. Each setting is a dictionary.
            - See U(https://docs.microsoft.com/en-us/azure/app-service/app-service-web-overview) for more info.
        type: list
        elements: dict
        suboptions:
            name:
                description:
                    - Name of the framework.
                    - Supported framework list for Windows web app and Linux web app is different.
                    - Windows web apps support C(java), C(net_framework), C(php), C(python), and C(node) from June 2018.
                    - Windows web apps support multiple framework at same time.
                    - Linux web apps support C(java), C(ruby), C(php), C(dotnetcore), and C(node) from June 2018.
                    - Linux web apps support only one framework.
                    - Java framework is mutually exclusive with others.
                type: str
                required: true
                choices:
                    - java
                    - net_framework
                    - php
                    - python
                    - ruby
                    - dotnetcore
                    - node
            version:
                description:
                    - Version of the framework. For Linux web app supported value, see U(https://aka.ms/linux-stacks) for more info.
                    - C(net_framework) supported value sample, C(v4.0) for .NET 4.6 and C(v3.0) for .NET 3.5.
                    - C(php) supported value sample, C(5.5), C(5.6), C(7.0).
                    - C(python) supported value sample, C(5.5), C(5.6), C(7.0).
                    - C(node) supported value sample, C(6.6), C(6.9).
                    - C(dotnetcore) supported value sample, C(1.0), C(1.1), C(1.2).
                    - C(ruby) supported value sample, 2.3.
                    - C(java) supported value sample, C(1.9) for Windows web app. C(1.8) for Linux web app.
                type: str
                required: true
            settings:
                description:
                    - List of settings of the framework.
                type: dict
                suboptions:
                    java_container:
                        description:
                            - Name of Java container. This is supported by specific framework C(java) onlys, for example C(Tomcat), C(Jetty).
                        type: str
                        required: true
                    java_container_version:
                        description:
                            - Version of Java container. This is supported by specific framework C(java) only.
                            - For C(Tomcat), for example C(8.0), C(8.5), C(9.0). For C(Jetty), for example C(9.1), C(9.3).
                        type: str
                        required: true
    container_settings:
        description:
            - Web app slot container settings.
        type: dict
        suboptions:
            name:
                description:
                    - Name of container, for example C(imagename:tag).
                type: str
                required: true
            registry_server_url:
                description:
                    - Container registry server URL, for example C(mydockerregistry.io).
                type: str
            registry_server_user:
                description:
                    - The container registry server user name.
                type: str
            registry_server_password:
                description:
                    - The container registry server password.
                type: str
    startup_file:
        description:
            - The slot startup file.
            - This only applies for Linux web app slot.
        type: str
    app_settings:
        description:
            - Configure web app slot application settings. Suboptions are in key value pair format.
        type: dict
    purge_app_settings:
        description:
            - Purge any existing application settings. Replace slot application settings with app_settings.
        type: bool
        default: False
    deployment_source:
        description:
            - Deployment source for git.
        type: dict
        suboptions:
            url:
                description:
                    - Repository URL of deployment source.
                type: str
            branch:
                description:
                    - The branch name of the repository.
                type: str
    app_state:
        description:
            - Start/Stop/Restart the slot.
        type: str
        choices:
            - started
            - stopped
            - restarted
        default: started
    state:
        description:
            - State of the Web App deployment slot.
            - Use C(present) to create or update a  slot and C(absent) to delete it.
        default: present
        type: str
        choices:
            - absent
            - present

extends_documentation_fragment:
    - azure.azcollection.azure
    - azure.azcollection.azure_tags

author:
    - Yunge Zhu(@yungezz)

'''

EXAMPLES = '''
- name: Create a webapp slot
  azure_rm_webappslot:
    resource_group: myResourceGroup
    webapp_name: myJavaWebApp
    name: stage
    configuration_source: myJavaWebApp
    app_settings:
      testkey: testvalue

- name: swap the slot with production slot
  azure_rm_webappslot:
    resource_group: myResourceGroup
    webapp_name: myJavaWebApp
    name: stage
    swap:
      action: swap

- name: stop the slot
  azure_rm_webappslot:
    resource_group: myResourceGroup
    webapp_name: myJavaWebApp
    name: stage
    app_state: stopped

- name: udpate a webapp slot app settings
  azure_rm_webappslot:
    resource_group: myResourceGroup
    webapp_name: myJavaWebApp
    name: stage
    app_settings:
      testkey: testvalue2

- name: udpate a webapp slot frameworks
  azure_rm_webappslot:
    resource_group: myResourceGroup
    webapp_name: myJavaWebApp
    name: stage
    frameworks:
      - name: "node"
        version: "10.1"
'''

RETURN = '''
id:
    description:
        - ID of current slot.
    returned: always
    type: str
    sample: /subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/myResourceGroup/providers/Microsoft.Web/sites/testapp/slots/stage1
'''

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase

try:
    from azure.core.exceptions import ResourceNotFoundError
    from azure.core.polling import LROPoller
    from azure.mgmt.web.models import Site, NameValuePair, SiteSourceControl, CsmSlotEntity, StringDictionary
except ImportError:
    # This is handled in azure_rm_common
    pass

swap_spec = dict(
    action=dict(
        type='str',
        choices=[
            'preview',
            'swap',
            'reset'
        ],
        default='preview'
    ),
    target_slot=dict(
        type='str'
    ),
    preserve_vnet=dict(
        type='bool',
        default=True
    )
)

container_settings_spec = dict(
    name=dict(type='str', required=True),
    registry_server_url=dict(type='str'),
    registry_server_user=dict(type='str'),
    registry_server_password=dict(type='str', no_log=True)
)

deployment_source_spec = dict(
    url=dict(type='str'),
    branch=dict(type='str')
)


framework_settings_spec = dict(
    java_container=dict(type='str', required=True),
    java_container_version=dict(type='str', required=True)
)


framework_spec = dict(
    name=dict(
        type='str',
        required=True,
        choices=['net_framework', 'java', 'php', 'node', 'python', 'dotnetcore', 'ruby']),
    version=dict(type='str', required=True),
    settings=dict(type='dict', options=framework_settings_spec)
)


def webapp_to_dict(webapp):
    return dict(
        id=webapp.id,
        name=webapp.name,
        location=webapp.location,
        client_cert_enabled=webapp.client_cert_enabled,
        enabled=webapp.enabled,
        reserved=webapp.reserved,
        client_affinity_enabled=webapp.client_affinity_enabled,
        server_farm_id=webapp.server_farm_id,
        host_names_disabled=webapp.host_names_disabled,
        https_only=webapp.https_only if hasattr(webapp, 'https_only') else None,
        skip_custom_domain_verification=webapp.skip_custom_domain_verification if hasattr(webapp, 'skip_custom_domain_verification') else None,
        ttl_in_seconds=webapp.ttl_in_seconds if hasattr(webapp, 'ttl_in_seconds') else None,
        state=webapp.state,
        tags=webapp.tags if webapp.tags else None
    )


def slot_to_dict(slot):
    return dict(
        id=slot.id,
        resource_group=slot.resource_group,
        server_farm_id=slot.server_farm_id,
        target_swap_slot=slot.target_swap_slot,
        enabled_host_names=slot.enabled_host_names,
        slot_swap_status=slot.slot_swap_status,
        name=slot.name,
        location=slot.location,
        enabled=slot.enabled,
        reserved=slot.reserved,
        host_names_disabled=slot.host_names_disabled,
        state=slot.state,
        repository_site_name=slot.repository_site_name,
        default_host_name=slot.default_host_name,
        kind=slot.kind,
        site_config=slot.site_config,
        tags=slot.tags if slot.tags else None
    )


class Actions:
    NoAction, CreateOrUpdate, UpdateAppSettings, Delete = range(4)


class AzureRMWebAppSlots(AzureRMModuleBase):
    """Configuration class for an Azure RM Web App slot resource"""

    def __init__(self):
        self.module_arg_spec = dict(
            resource_group=dict(
                type='str',
                required=True
            ),
            name=dict(
                type='str',
                required=True
            ),
            webapp_name=dict(
                type='str',
                required=True
            ),
            location=dict(
                type='str'
            ),
            configuration_source=dict(
                type='str'
            ),
            auto_swap_slot_name=dict(
                type='raw'
            ),
            swap=dict(
                type='dict',
                options=swap_spec
            ),
            frameworks=dict(
                type='list',
                elements='dict',
                options=framework_spec
            ),
            container_settings=dict(
                type='dict',
                options=container_settings_spec
            ),
            deployment_source=dict(
                type='dict',
                options=deployment_source_spec
            ),
            startup_file=dict(
                type='str'
            ),
            app_settings=dict(
                type='dict'
            ),
            purge_app_settings=dict(
                type='bool',
                default=False
            ),
            app_state=dict(
                type='str',
                choices=['started', 'stopped', 'restarted'],
                default='started'
            ),
            state=dict(
                type='str',
                default='present',
                choices=['present', 'absent']
            )
        )

        mutually_exclusive = [['container_settings', 'frameworks']]

        self.resource_group = None
        self.name = None
        self.webapp_name = None
        self.location = None

        self.auto_swap_slot_name = None
        self.swap = None
        self.tags = None
        self.startup_file = None
        self.configuration_source = None
        self.clone = False

        # site config, e.g app settings, ssl
        self.site_config = dict()
        self.app_settings = dict()
        self.app_settings_strDic = None

        # siteSourceControl
        self.deployment_source = dict()

        # site, used at level creation, or update.
        self.site = None

        # property for internal usage, not used for sdk
        self.container_settings = None

        self.purge_app_settings = False
        self.app_state = 'started'

        self.results = dict(
            changed=False,
            id=None,
        )
        self.state = None
        self.to_do = Actions.NoAction

        self.frameworks = None

        # set site_config value from kwargs
        self.site_config_updatable_frameworks = ["net_framework_version",
                                                 "java_version",
                                                 "php_version",
                                                 "python_version",
                                                 "linux_fx_version"]

        self.supported_linux_frameworks = ['ruby', 'php', 'dotnetcore', 'node', 'java']
        self.supported_windows_frameworks = ['net_framework', 'php', 'python', 'node', 'java']

        super(AzureRMWebAppSlots, self).__init__(derived_arg_spec=self.module_arg_spec,
                                                 mutually_exclusive=mutually_exclusive,
                                                 supports_check_mode=True,
                                                 supports_tags=True)

    def exec_module(self, **kwargs):
        """Main module execution method"""

        for key in list(self.module_arg_spec.keys()) + ['tags']:
            if hasattr(self, key):
                setattr(self, key, kwargs[key])
            elif kwargs[key] is not None:
                if key == "scm_type":
                    self.site_config[key] = kwargs[key]

        old_response = None
        response = None
        to_be_updated = False

        # set location
        resource_group = self.get_resource_group(self.resource_group)
        if not self.location:
            self.location = resource_group.location

        # get web app
        webapp_response = self.get_webapp()

        if not webapp_response:
            self.fail("Web app {0} does not exist in resource group {1}.".format(self.webapp_name, self.resource_group))

        # get slot
        old_response = self.get_slot()

        # set is_linux
        is_linux = True if webapp_response['reserved'] else False

        if self.state == 'present':
            if self.frameworks:
                # java is mutually exclusive with other frameworks
                if len(self.frameworks) > 1 and any(f['name'] == 'java' for f in self.frameworks):
                    self.fail('Java is mutually exclusive with other frameworks.')

                if is_linux:
                    if len(self.frameworks) != 1:
                        self.fail('Can specify one framework only for Linux web app.')

                    if self.frameworks[0]['name'] not in self.supported_linux_frameworks:
                        self.fail('Unsupported framework {0} for Linux web app.'.format(self.frameworks[0]['name']))

                    self.site_config['linux_fx_version'] = (self.frameworks[0]['name'] + '|' + self.frameworks[0]['version']).upper()

                    if self.frameworks[0]['name'] == 'java':
                        if self.frameworks[0]['version'] != '8':
                            self.fail("Linux web app only supports java 8.")

                        if self.frameworks[0].get('settings', {}) and self.frameworks[0]['settings'].get('java_container', None) and \
                           self.frameworks[0]['settings']['java_container'].lower() != 'tomcat':
                            self.fail("Linux web app only supports tomcat container.")

                        if self.frameworks[0].get('settings', {}) and self.frameworks[0]['settings'].get('java_container', None) and \
                           self.frameworks[0]['settings']['java_container'].lower() == 'tomcat':
                            self.site_config['linux_fx_version'] = 'TOMCAT|' + self.frameworks[0]['settings']['java_container_version'] + '-jre8'
                        else:
                            self.site_config['linux_fx_version'] = 'JAVA|8-jre8'
                else:
                    for fx in self.frameworks:
                        if fx.get('name') not in self.supported_windows_frameworks:
                            self.fail('Unsupported framework {0} for Windows web app.'.format(fx.get('name')))
                        else:
                            self.site_config[fx.get('name') + '_version'] = fx.get('version')

                        if 'settings' in fx and fx['settings'] is not None:
                            for key, value in fx['settings'].items():
                                self.site_config[key] = value

            if not self.app_settings:
                self.app_settings = dict()

            if self.container_settings:
                linux_fx_version = 'DOCKER|'

                if self.container_settings.get('registry_server_url'):
                    self.app_settings['DOCKER_REGISTRY_SERVER_URL'] = 'https://' + self.container_settings['registry_server_url']

                    linux_fx_version += self.container_settings['registry_server_url'] + '/'

                linux_fx_version += self.container_settings['name']

                self.site_config['linux_fx_version'] = linux_fx_version

                if self.container_settings.get('registry_server_user'):
                    self.app_settings['DOCKER_REGISTRY_SERVER_USERNAME'] = self.container_settings['registry_server_user']

                if self.container_settings.get('registry_server_password'):
                    self.app_settings['DOCKER_REGISTRY_SERVER_PASSWORD'] = self.container_settings['registry_server_password']

            # set auto_swap_slot_name
            if self.auto_swap_slot_name and isinstance(self.auto_swap_slot_name, str):
                self.site_config['auto_swap_slot_name'] = self.auto_swap_slot_name
            if self.auto_swap_slot_name is False:
                self.site_config['auto_swap_slot_name'] = None

            # init site
            self.site = Site(location=self.location, site_config=self.site_config)

            # check if the slot already present in the webapp
            if not old_response:
                self.log("Web App slot doesn't exist")

                to_be_updated = True
                self.to_do = Actions.CreateOrUpdate
                self.site.tags = self.tags

                # if linux, setup startup_file
                if self.startup_file:
                    self.site_config['app_command_line'] = self.startup_file

                # set app setting
                if self.app_settings:
                    app_settings = []
                    for key in self.app_settings.keys():
                        app_settings.append(NameValuePair(name=key, value=self.app_settings[key]))

                    self.site_config['app_settings'] = app_settings

                # clone slot
                if self.configuration_source:
                    self.clone = True

            else:
                # existing slot, do update
                self.log("Web App slot already exists")

                self.log('Result: {0}'.format(old_response))

                update_tags, self.site.tags = self.update_tags(old_response.get('tags', None))

                if update_tags:
                    to_be_updated = True

                # check if site_config changed
                old_config = self.get_configuration_slot(self.name)

                if self.is_site_config_changed(old_config):
                    to_be_updated = True
                    self.to_do = Actions.CreateOrUpdate

                self.app_settings_strDic = self.list_app_settings_slot(self.name)

                # purge existing app_settings:
                if self.purge_app_settings:
                    to_be_updated = True
                    self.to_do = Actions.UpdateAppSettings
                    self.app_settings_strDic = dict()

                # check if app settings changed
                if self.purge_app_settings or self.is_app_settings_changed():
                    to_be_updated = True
                    self.to_do = Actions.UpdateAppSettings

                    if self.app_settings:
                        for key in self.app_settings.keys():
                            self.app_settings_strDic[key] = self.app_settings[key]

        elif self.state == 'absent':
            if old_response:
                self.log("Delete Web App slot")
                self.results['changed'] = True

                if self.check_mode:
                    return self.results

                self.delete_slot()

                self.log('Web App slot deleted')

            else:
                self.log("Web app slot {0} not exists.".format(self.name))

        if to_be_updated:
            self.log('Need to Create/Update web app')
            self.results['changed'] = True

            if self.check_mode:
                return self.results

            if self.to_do == Actions.CreateOrUpdate:
                response = self.create_update_slot()

                self.results['id'] = response['id']

                if self.clone:
                    self.clone_slot()

            if self.to_do == Actions.UpdateAppSettings:
                self.update_app_settings_slot()

        slot = None
        if response:
            slot = response
        if old_response:
            slot = old_response

        if slot:
            if (slot['state'] != 'Stopped' and self.app_state == 'stopped') or \
               (slot['state'] != 'Running' and self.app_state == 'started') or \
               self.app_state == 'restarted':

                self.results['changed'] = True
                if self.check_mode:
                    return self.results

                self.set_state_slot(self.app_state)

            if self.swap:
                self.results['changed'] = True
                if self.check_mode:
                    return self.results

                self.swap_slot()

        return self.results

    # compare site config
    def is_site_config_changed(self, existing_config):
        for fx_version in self.site_config_updatable_frameworks:
            if self.site_config.get(fx_version):
                if not getattr(existing_config, fx_version) or \
                        getattr(existing_config, fx_version).upper() != self.site_config.get(fx_version).upper():
                    return True

        if self.auto_swap_slot_name is False and existing_config.auto_swap_slot_name is not None:
            return True
        elif self.auto_swap_slot_name and self.auto_swap_slot_name != getattr(existing_config, 'auto_swap_slot_name', None):
            return True
        return False

    # comparing existing app setting with input, determine whether it's changed
    def is_app_settings_changed(self):
        if self.app_settings:
            if len(self.app_settings_strDic) != len(self.app_settings):
                return True

            if self.app_settings_strDic != self.app_settings:
                return True
        return False

    # comparing deployment source with input, determine whether it's changed
    def is_deployment_source_changed(self, existing_webapp):
        if self.deployment_source:
            if self.deployment_source.get('url') \
                    and self.deployment_source['url'] != existing_webapp.get('site_source_control')['url']:
                return True

            if self.deployment_source.get('branch') \
                    and self.deployment_source['branch'] != existing_webapp.get('site_source_control')['branch']:
                return True

        return False

    def create_update_slot(self):
        '''
        Creates or updates Web App slot with the specified configuration.

        :return: deserialized Web App instance state dictionary
        '''
        self.log(
            "Creating / Updating the Web App slot {0}".format(self.name))

        try:
            response = self.web_client.web_apps.begin_create_or_update_slot(resource_group_name=self.resource_group,
                                                                            slot=self.name,
                                                                            name=self.webapp_name,
                                                                            site_envelope=self.site)
            if isinstance(response, LROPoller):
                response = self.get_poller_result(response)

        except Exception as exc:
            self.log('Error attempting to create the Web App slot instance.')
            self.fail("Error creating the Web App slot: {0}".format(str(exc)))
        return slot_to_dict(response)

    def delete_slot(self):
        '''
        Deletes specified Web App slot in the specified subscription and resource group.

        :return: True
        '''
        self.log("Deleting the Web App slot {0}".format(self.name))
        try:
            self.web_client.web_apps.delete_slot(resource_group_name=self.resource_group,
                                                 name=self.webapp_name,
                                                 slot=self.name)
        except Exception as e:
            self.log('Error attempting to delete the Web App slot.')
            self.fail(
                "Error deleting the Web App slots: {0}".format(str(e)))

        return True

    def get_webapp(self):
        '''
        Gets the properties of the specified Web App.

        :return: deserialized Web App instance state dictionary
        '''
        self.log(
            "Checking if the Web App instance {0} is present".format(self.webapp_name))

        response = None

        try:
            response = self.web_client.web_apps.get(resource_group_name=self.resource_group,
                                                    name=self.webapp_name)

            # Newer SDK versions (0.40.0+) seem to return None if it doesn't exist instead of raising error
            if response is not None:
                self.log("Response : {0}".format(response))
                self.log("Web App instance : {0} found".format(response.name))
                return webapp_to_dict(response)

        except ResourceNotFoundError:
            pass

        self.log("Didn't find web app {0} in resource group {1}".format(
            self.webapp_name, self.resource_group))

        return False

    def get_slot(self):
        '''
        Gets the properties of the specified Web App slot.

        :return: deserialized Web App slot state dictionary
        '''
        self.log(
            "Checking if the Web App slot {0} is present".format(self.name))

        response = None

        try:
            response = self.web_client.web_apps.get_slot(resource_group_name=self.resource_group,
                                                         name=self.webapp_name,
                                                         slot=self.name)

            # Newer SDK versions (0.40.0+) seem to return None if it doesn't exist instead of raising error
            if response is not None:
                self.log("Response : {0}".format(response))
                self.log("Web App slot: {0} found".format(response.name))
                return slot_to_dict(response)

        except ResourceNotFoundError:
            pass

        self.log("Does not find web app slot {0} in resource group {1}".format(self.name, self.resource_group))

        return False

    def list_app_settings(self):
        '''
        List webapp application settings
        :return: deserialized list response
        '''
        self.log("List webapp application setting")

        try:
            response = self.web_client.web_apps.list_application_settings(
                resource_group_name=self.resource_group, name=self.webapp_name)
            self.log("Response : {0}".format(response))

            return response.properties
        except Exception as ex:
            self.fail("Failed to list application settings for web app {0} in resource group {1}: {2}".format(
                self.name, self.resource_group, str(ex)))

    def list_app_settings_slot(self, slot_name):
        '''
        List application settings
        :return: deserialized list response
        '''
        self.log("List application setting")

        try:
            response = self.web_client.web_apps.list_application_settings_slot(
                resource_group_name=self.resource_group, name=self.webapp_name, slot=slot_name)
            self.log("Response : {0}".format(response))

            return response.properties
        except Exception as ex:
            self.fail("Failed to list application settings for web app slot {0} in resource group {1}: {2}".format(
                self.name, self.resource_group, str(ex)))

    def update_app_settings_slot(self, slot_name=None, app_settings=None):
        '''
        Update application settings
        :return: deserialized updating response
        '''
        self.log("Update application setting")

        if slot_name is None:
            slot_name = self.name
        if app_settings is None:
            app_settings = self.app_settings_strDic
        try:
            settings = StringDictionary(
                properties=self.app_settings
            )
            response = self.web_client.web_apps.update_application_settings_slot(resource_group_name=self.resource_group,
                                                                                 name=self.webapp_name,
                                                                                 slot=slot_name,
                                                                                 app_settings=settings)
            self.log("Response : {0}".format(response))

            return response.as_dict()
        except Exception as ex:
            self.fail("Failed to update application settings for web app slot {0} in resource group {1}: {2}".format(
                self.name, self.resource_group, str(ex)))

        return response

    def create_or_update_source_control_slot(self):
        '''
        Update site source control
        :return: deserialized updating response
        '''
        self.log("Update site source control")

        if self.deployment_source is None:
            return False

        self.deployment_source['is_manual_integration'] = False
        self.deployment_source['is_mercurial'] = False

        try:
            site_source_control = SiteSourceControl(
                repo_url=self.deployment_source.get('url'),
                branch=self.deployment_source.get('branch')
            )
            response = self.web_client.web_client.begin_create_or_update_source_control_slot(
                resource_group_name=self.resource_group,
                name=self.webapp_name,
                site_source_control=site_source_control,
                slot=self.name)
            self.log("Response : {0}".format(response))

            return response.as_dict()
        except Exception as ex:
            self.fail("Failed to update site source control for web app slot {0} in resource group {1}: {2}".format(
                self.name, self.resource_group, str(ex)))

    def get_configuration(self):
        '''
        Get web app configuration
        :return: deserialized web app configuration response
        '''
        self.log("Get web app configuration")

        try:
            response = self.web_client.web_apps.get_configuration(
                resource_group_name=self.resource_group, name=self.webapp_name)
            self.log("Response : {0}".format(response))

            return response
        except Exception as ex:
            self.fail("Failed to get configuration for web app {0} in resource group {1}: {2}".format(
                self.webapp_name, self.resource_group, str(ex)))

    def get_configuration_slot(self, slot_name):
        '''
        Get slot configuration
        :return: deserialized slot configuration response
        '''
        self.log("Get web app slot configuration")

        try:
            response = self.web_client.web_apps.get_configuration_slot(
                resource_group_name=self.resource_group, name=self.webapp_name, slot=slot_name)
            self.log("Response : {0}".format(response))

            return response
        except Exception as ex:
            self.fail("Failed to get configuration for web app slot {0} in resource group {1}: {2}".format(
                slot_name, self.resource_group, str(ex)))

    def update_configuration_slot(self, slot_name=None, site_config=None):
        '''
        Update slot configuration
        :return: deserialized slot configuration response
        '''
        self.log("Update web app slot configuration")

        if slot_name is None:
            slot_name = self.name
        if site_config is None:
            site_config = self.site_config
        try:
            response = self.web_client.web_apps.update_configuration_slot(
                resource_group_name=self.resource_group, name=self.webapp_name, slot=slot_name, site_config=site_config)
            self.log("Response : {0}".format(response))

            return response
        except Exception as ex:
            self.fail("Failed to update configuration for web app slot {0} in resource group {1}: {2}".format(
                slot_name, self.resource_group, str(ex)))

    def set_state_slot(self, appstate):
        '''
        Start/stop/restart web app slot
        :return: deserialized updating response
        '''
        try:
            if appstate == 'started':
                response = self.web_client.web_apps.start_slot(resource_group_name=self.resource_group, name=self.webapp_name, slot=self.name)
            elif appstate == 'stopped':
                response = self.web_client.web_apps.stop_slot(resource_group_name=self.resource_group, name=self.webapp_name, slot=self.name)
            elif appstate == 'restarted':
                response = self.web_client.web_apps.restart_slot(resource_group_name=self.resource_group, name=self.webapp_name, slot=self.name)
            else:
                self.fail("Invalid web app slot state {0}".format(appstate))

            self.log("Response : {0}".format(response))

            return response
        except Exception as ex:
            request_id = ex.request_id if ex.request_id else ''
            self.fail("Failed to {0} web app slot {1} in resource group {2}, request_id {3} - {4}".format(
                appstate, self.name, self.resource_group, request_id, str(ex)))

    def swap_slot(self):
        '''
        Swap slot
        :return: deserialized response
        '''
        self.log("Swap slot")

        try:
            if self.swap['action'] == 'swap':
                if self.swap['target_slot'] is None:
                    slot_swap_entity = CsmSlotEntity(
                        target_slot=self.name,
                        preserve_vnet=self.swap['preserve_vnet']
                    )
                    response = self.web_client.web_apps.begin_swap_slot_with_production(resource_group_name=self.resource_group,
                                                                                        name=self.webapp_name,
                                                                                        slot_swap_entity=slot_swap_entity)
                else:
                    slot_swap_entity = CsmSlotEntity(
                        target_slot=self.swap['target_slot'],
                        preserve_vnet=self.swap['preserve_vnet']
                    )
                    response = self.web_client.web_apps.begin_swap_slot(resource_group_name=self.resource_group,
                                                                        name=self.webapp_name,
                                                                        slot=self.name,
                                                                        slot_swap_entity=slot_swap_entity)
            elif self.swap['action'] == 'preview':
                if self.swap['target_slot'] is None:
                    slot_swap_entity = CsmSlotEntity(
                        target_slot=self.name,
                        preserve_vnet=self.swap['preserve_vnet']
                    )
                    response = self.web_client.web_apps.apply_slot_config_to_production(resource_group_name=self.resource_group,
                                                                                        name=self.webapp_name,
                                                                                        slot_swap_entity=slot_swap_entity)
                else:
                    slot_swap_entity = CsmSlotEntity(
                        target_slot=self.swap['target_slot'],
                        preserve_vnet=self.swap['preserve_vnet']
                    )
                    response = self.web_client.web_apps.apply_slot_configuration_slot(resource_group_name=self.resource_group,
                                                                                      name=self.webapp_name,
                                                                                      slot=self.name,
                                                                                      slot_swap_entity=slot_swap_entity)
            elif self.swap['action'] == 'reset':
                if self.swap['target_slot'] is None:
                    response = self.web_client.web_apps.reset_production_slot_config(resource_group_name=self.resource_group,
                                                                                     name=self.webapp_name)
                else:
                    response = self.web_client.web_apps.reset_slot_configuration_slot(resource_group_name=self.resource_group,
                                                                                      name=self.webapp_name,
                                                                                      slot=self.swap['target_slot'])
                response = self.web_client.web_apps.reset_slot_configuration_slot(resource_group_name=self.resource_group,
                                                                                  name=self.webapp_name,
                                                                                  slot=self.name)

            self.log("Response : {0}".format(response))

            return response
        except Exception as ex:
            self.fail("Failed to swap web app slot {0} in resource group {1}: {2}".format(self.name, self.resource_group, str(ex)))

    def clone_slot(self):
        if self.configuration_source:
            src_slot = None if self.configuration_source.lower() == self.webapp_name.lower() else self.configuration_source

            if src_slot is None:
                site_config_clone_from = self.get_configuration()
            else:
                site_config_clone_from = self.get_configuration_slot(slot_name=src_slot)

            self.update_configuration_slot(site_config=site_config_clone_from)

            if src_slot is None:
                app_setting_clone_from = self.list_app_settings()
            else:
                app_setting_clone_from = self.list_app_settings_slot(src_slot)

            if self.app_settings:
                app_setting_clone_from.update(self.app_settings)

            self.update_app_settings_slot(app_settings=app_setting_clone_from)


def main():
    """Main execution"""
    AzureRMWebAppSlots()


if __name__ == '__main__':
    main()



#!/usr/bin/python
#
# Copyright (c) 2024 xuzhang3 (@xuzhang3), Fred-sun (@Fred-sun)
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_mysqlflexibleconfiguration
version_added: "2.7.0"
short_description: Manage Configuration instance
description:
    - Update or delete instance of Flexible Configuration.

options:
    resource_group:
        description:
            - The name of the resource group that contains the resource.
        required: True
        type: str
    server_name:
        description:
            - The name of the server.
        required: True
        type: str
    name:
        description:
            - The name of the server configuration.
        required: True
        type: str
    value:
        description:
            - The value server configurations.
        type: str
    source:
        description:
            - Source of the configuration.
        type: str
        choices:
            - system-default
            - user-override
extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - xuzhang3 (@xuzhang3)
    - Fred-sun (@Fred-sun)

'''

EXAMPLES = '''
- name: Update SQL Server setting
  azure_rm_mysqlflexibleconfiguration:
    resource_group: myResourceGroup
    server_name: myServer
    name: event_scheduler
    value:
      - name: testvalue
        value: "ON"
        source: system-default
'''

RETURN = '''
id:
    description:
        - Resource ID.
    returned: always
    type: str
    sample: "/subscriptions/xx-xx/resourceGroups/myRG/providers/Microsoft.DBforMySQL/flexibleservers/myServer/configurations/event_scheduler"
server_name:
    description:
        - The MySQL flexible server name.
    type: str
    returned: always
    sample: testmysqlserver
name:
    description:
        - Setting name.
    returned: always
    type: str
    sample: deadlock_timeout
value:
    description:
        - Setting value.
    returned: always
    type: raw
    sample: 1000
source:
    description:
        - Source of the configuration.
    returned: always
    type: str
    sample: system-default
'''

try:
    from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase
    from azure.core.exceptions import ResourceNotFoundError
    from azure.core.polling import LROPoller
except ImportError:
    # This is handled in azure_rm_common
    pass


class Actions:
    NoAction, Update = range(2)


class AzureRMMySqlFlexibleConfiguration(AzureRMModuleBase):

    def __init__(self):
        self.module_arg_spec = dict(
            resource_group=dict(
                type='str',
                required=True
            ),
            server_name=dict(
                type='str',
                required=True
            ),
            name=dict(
                type='str',
                required=True
            ),
            value=dict(type='str'),
            source=dict(type='str', choices=['system-default', 'user-override']),
        )

        self.resource_group = None
        self.server_name = None
        self.name = None
        self.value = None
        self.source = None

        self.results = dict(changed=False)
        self.to_do = Actions.NoAction

        super(AzureRMMySqlFlexibleConfiguration, self).__init__(derived_arg_spec=self.module_arg_spec,
                                                                supports_check_mode=True,
                                                                supports_tags=False)

    def exec_module(self, **kwargs):

        for key in list(self.module_arg_spec.keys()):
            if hasattr(self, key):
                setattr(self, key, kwargs[key])

        old_response = None
        response = None

        old_response = self.get_configuration()

        if not old_response:
            self.fail("The MySQL configuration not exist, We can't make any changes")
        else:
            self.log("Need to check if Configuration instance has to be deleted or may be updated")
            if self.value is not None and self.value != old_response['value'] or self.source is not None and self.source != old_response['source']:
                self.to_do = Actions.Update

        if self.to_do == Actions.Update:
            self.log("Need to Update the Configuration instance")
            if not self.check_mode:
                response = self.update_configuration()
                self.results['changed'] = True
                self.log("Update done")
        response = self.get_configuration()

        if response:
            self.results["id"] = response["id"]
            self.results['resource_group'] = self.resource_group
            self.results['server_name'] = self.server_name
            self.results['name'] = self.name
            self.results['source'] = response['source']
            self.results['value'] = response['value']

        return self.results

    def update_configuration(self):
        self.log("Updating the Configuration instance {0}".format(self.name))

        try:
            response = self.mysql_flexible_client.configurations.begin_update(resource_group_name=self.resource_group,
                                                                              server_name=self.server_name,
                                                                              configuration_name=self.name,
                                                                              parameters={'value': self.value, 'source': self.source})
            if isinstance(response, LROPoller):
                response = self.get_poller_result(response)

        except Exception as exc:
            self.log('Error attempting to create the Configuration instance.')
            self.fail("Error creating the Configuration instance: {0}".format(str(exc)))
        return response.as_dict()

    def get_configuration(self):
        self.log("Checking if the Configuration instance {0} is present".format(self.name))
        found = False
        try:
            response = self.mysql_flexible_client.configurations.get(resource_group_name=self.resource_group,
                                                                     server_name=self.server_name,
                                                                     configuration_name=self.name)
            found = True
            self.log("Response : {0}".format(response))
            self.log("Configuration instance : {0} found".format(response.name))
        except ResourceNotFoundError as e:
            self.log('Did not find the Configuration instance.')
        if found is True:
            return response.as_dict()

        return False


def main():
    """Main execution"""
    AzureRMMySqlFlexibleConfiguration()


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
# Copyright 2024 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function


__metaclass__ = type

"""
The ios vrf_global fact class
It is in this file the configuration is collected from the device
for a given resource, parsed, and the facts tree is populated
based on the configuration.
"""

import re

from ansible_collections.ansible.netcommon.plugins.module_utils.network.common import utils

from ansible_collections.cisco.ios.plugins.module_utils.network.ios.argspec.vrf_global.vrf_global import (
    Vrf_globalArgs,
)
from ansible_collections.cisco.ios.plugins.module_utils.network.ios.rm_templates.vrf_global import (
    Vrf_globalTemplate,
)


class Vrf_globalFacts(object):
    """The ios vrf_global facts class"""

    def __init__(self, module, subspec="config", options="options"):
        self._module = module
        self.argument_spec = Vrf_globalArgs.argument_spec

    def get_config(self, connection):
        """Get the configuration from the device"""
        raw_config = connection.get("show running-config | section ^vrf definition")
        return self._filter_global_config(raw_config)

    def _filter_global_config(self, config_text):
        """
        Filter out address-family specific configuration from VRF config.
        Only return global VRF configuration lines.
        """
        if not config_text:
            return ""

        lines = config_text.splitlines()
        filtered_lines = []
        inside_address_family = False

        for line in lines:
            if re.match(r"\s+address-family\s+", line):
                inside_address_family = True
                continue

            if re.match(r"\s+exit-address-family", line):
                inside_address_family = False
                continue

            if not inside_address_family:
                filtered_lines.append(line)

        return "\n".join(filtered_lines)

    def populate_facts(self, connection, ansible_facts, data=None):
        """Populate the facts for Vrf_global network resource

        :param connection: the device connection
        :param ansible_facts: Facts dictionary
        :param data: previously collected conf

        :rtype: dictionary
        :returns: facts
        """

        facts = {}
        objs = []

        if not data:
            data = self.get_config(connection)

        vrf_global_parser = Vrf_globalTemplate(lines=data.splitlines(), module=self._module)
        objs = vrf_global_parser.parse()

        objs["vrfs"] = list(objs["vrfs"].values()) if "vrfs" in objs else []

        ansible_facts["ansible_network_resources"].pop("vrf_global", None)
        params = utils.remove_empties(
            vrf_global_parser.validate_config(self.argument_spec, {"config": objs}, redact=True),
        )

        facts["vrf_global"] = params.get("config", {})
        ansible_facts["ansible_network_resources"].update(facts)

        return ansible_facts



# Copyright (c) 2025, Marcos Alano <marcoshalano@gmail.com>
# Based on gio_mime module. Copyright (c) 2022, Alexei Znamensky <russoz@gmail.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

# Note that this module util is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

from __future__ import annotations

import typing as t

from ansible_collections.community.general.plugins.module_utils._cmd_runner import CmdRunner, cmd_runner_fmt

if t.TYPE_CHECKING:
    from ansible.module_utils.basic import AnsibleModule


def xdg_mime_runner(module: AnsibleModule, **kwargs) -> CmdRunner:
    return CmdRunner(
        module,
        command=["xdg-mime"],
        arg_formats=dict(
            default=cmd_runner_fmt.as_fixed("default"),
            query=cmd_runner_fmt.as_fixed("query"),
            mime_types=cmd_runner_fmt.as_list(),
            handler=cmd_runner_fmt.as_list(),
            version=cmd_runner_fmt.as_fixed("--version"),
        ),
        **kwargs,
    )


def xdg_mime_get(runner: CmdRunner, mime_type) -> str | None:
    def process(rc, out, err) -> str | None:
        if not out.strip():
            return None
        out = out.splitlines()[0]
        return out.split()[-1]

    with runner("query default mime_types", output_process=process) as ctx:
        return ctx.run(mime_types=mime_type)
